@@ -42,6 +42,8 @@ type specific_operation =
   | Ibswap of int                      (* endianness conversion *)
   | Isqrtf                             (* Float square root *)
   | Ifloatsqrtf of addressing_mode     (* Float square root from memory *)
+  | Isextend32                         (* 32 to 64 bit conversion with sign
+                                          extension *)
 and float_operation =
     Ifloatadd | Ifloatsub | Ifloatmul | Ifloatdiv
 
@@ -126,6 +128,8 @@ let print_specific_operation printreg op ppf arg =
                    (Array.sub arg 1 (Array.length arg - 1))
   | Ibswap i ->
       fprintf ppf "bswap_%i %a" i printreg arg.(0)
+  | Isextend32 ->
+      fprintf ppf "sextend32 %a" printreg arg.(0)
 
 let win64 =
   match Config.system with
