@@ -61,7 +61,7 @@ module type S = sig
      relaxed generically.  It is assumed that these rewrites do not change
      the size of out-of-line code (cf. branch_relaxation.mli). *)
   val relax_allocation
-     : num_words:int
+     : num_bytes:int
     -> label_after_call_gc:Cmm.label option
     -> Linearize.instruction_desc
   val relax_intop_checkbound
