@@ -18,41 +18,42 @@
 open Mach
 
 type allocation_state =
-    No_alloc                            (* no allocation is pending *)
-  | Pending_alloc of Reg.t * int        (* an allocation is pending *)
-(* The arguments of Pending_alloc(reg, ofs) are:
-     reg  the register holding the result of the last allocation
-     ofs  the alloc position in the allocated block *)
+    No_alloc
+  | Pending_alloc of
+    { reg: Reg.t;    (* register holding the result of the last allocation *)
+      totalsz: int } (* amount to be allocated in this block *)
 
 let allocated_size = function
     No_alloc -> 0
-  | Pending_alloc(_, ofs) -> ofs
+  | Pending_alloc {totalsz; _} -> totalsz
 
 let rec combine i allocstate =
   match i.desc with
     Iend | Ireturn | Iexit _ | Iraise _ ->
       (i, allocated_size allocstate)
-  | Iop(Ialloc { words = sz; _ }) ->
+  | Iop(Ialloc { bytes = sz; _ }) ->
       begin match allocstate with
-        No_alloc ->
-          let (newnext, newsz) =
-            combine i.next (Pending_alloc(i.res.(0), sz)) in
-          (instr_cons_debug (Iop(Ialloc {words = newsz; spacetime_index = 0;
-              label_after_call_gc = None; }))
-            i.arg i.res i.dbg newnext, 0)
-      | Pending_alloc(reg, ofs) ->
-          if ofs + sz < Config.max_young_wosize * Arch.size_addr then begin
-            let (newnext, newsz) =
-              combine i.next (Pending_alloc(reg, ofs + sz)) in
-            (instr_cons (Iop(Iintop_imm(Iadd, ofs))) [| reg |] i.res newnext,
-             newsz)
-          end else begin
-            let (newnext, newsz) =
-              combine i.next (Pending_alloc(i.res.(0), sz)) in
-            (instr_cons_debug (Iop(Ialloc { words = newsz; spacetime_index = 0;
-                label_after_call_gc = None; }))
-              i.arg i.res i.dbg newnext, ofs)
-          end
+      | Pending_alloc {reg; totalsz}
+          when totalsz + sz < Config.max_young_wosize * Arch.size_addr ->
+         let (next, totalsz) =
+           combine i.next
+             (Pending_alloc { reg = i.res.(0); totalsz = totalsz + sz }) in
+         (instr_cons_debug (Iop(Iintop_imm(Iadd, -sz)))
+            [| reg |] i.res i.dbg next,
+          totalsz)
+      | No_alloc | Pending_alloc _ ->
+         let (next, totalsz) =
+           combine i.next
+             (Pending_alloc { reg = i.res.(0); totalsz = sz }) in
+         let next =
+           let offset = totalsz - sz in
+           if offset = 0 then next
+           else instr_cons_debug (Iop(Iintop_imm(Iadd, offset))) i.res
+                i.res i.dbg next
+         in
+         (instr_cons_debug (Iop(Ialloc {bytes = totalsz; spacetime_index = 0;
+                                        label_after_call_gc = None; }))
+          i.arg i.res i.dbg next, allocated_size allocstate)
       end
   | Iop(Icall_ind _ | Icall_imm _ | Iextcall _ |
         Itailcall_ind _ | Itailcall_imm _) ->
