@@ -16,28 +16,30 @@
 
 [@@@ocaml.warning "+a-4-9-30-40-41-42"]
 
-type for_one_or_more_units = {
+module V = Backend_var
+module VP = Backend_var.With_provenance
+
+type 'a for_one_or_more_units = {
   fun_offset_table : int Closure_id.Map.t;
   fv_offset_table : int Var_within_closure.Map.t;
-  closures : Flambda.function_declarations Closure_id.Map.t;
-  constant_sets_of_closures : Set_of_closures_id.Set.t;
+  constant_closures : Closure_id.Set.t;
+  closures: Closure_id.Set.t;
 }
 
 type t = {
-  current_unit : for_one_or_more_units;
-  imported_units : for_one_or_more_units;
+  current_unit :
+    Set_of_closures_id.t for_one_or_more_units;
+  imported_units :
+    Simple_value_approx.function_declarations for_one_or_more_units;
 }
 
-type ('a, 'b) declaration_position =
-  | Current_unit of 'a
-  | Imported_unit of 'b
-  | Not_declared
-
 let get_fun_offset t closure_id =
   let fun_offset_table =
     if Closure_id.in_compilation_unit closure_id (Compilenv.current_unit ())
-    then t.current_unit.fun_offset_table
-    else t.imported_units.fun_offset_table
+    then
+      t.current_unit.fun_offset_table
+    else
+      t.imported_units.fun_offset_table
   in
   try Closure_id.Map.find closure_id fun_offset_table
   with Not_found ->
@@ -56,23 +58,12 @@ let get_fv_offset t var_within_closure =
     Misc.fatal_errorf "Flambda_to_clambda: missing offset for variable %a"
       Var_within_closure.print var_within_closure
 
-let function_declaration_position t closure_id =
-  try
-    Current_unit (Closure_id.Map.find closure_id t.current_unit.closures)
-  with Not_found ->
-    try
-      Imported_unit (Closure_id.Map.find closure_id t.imported_units.closures)
-    with Not_found -> Not_declared
-
 let is_function_constant t closure_id =
-  match function_declaration_position t closure_id with
-  | Current_unit { set_of_closures_id } ->
-    Set_of_closures_id.Set.mem set_of_closures_id
-      t.current_unit.constant_sets_of_closures
-  | Imported_unit { set_of_closures_id } ->
-    Set_of_closures_id.Set.mem set_of_closures_id
-      t.imported_units.constant_sets_of_closures
-  | Not_declared ->
+  if Closure_id.Set.mem closure_id t.current_unit.closures then
+    Closure_id.Set.mem closure_id t.current_unit.constant_closures
+  else if Closure_id.Set.mem closure_id t.imported_units.closures then
+    Closure_id.Set.mem closure_id t.imported_units.constant_closures
+  else
     Misc.fatal_errorf "Flambda_to_clambda: missing closure %a"
       Closure_id.print closure_id
 
@@ -121,11 +112,11 @@ module Env : sig
   val add_subst : t -> Variable.t -> Clambda.ulambda -> t
   val find_subst_exn : t -> Variable.t -> Clambda.ulambda
 
-  val add_fresh_ident : t -> Variable.t -> Ident.t * t
-  val ident_for_var_exn : t -> Variable.t -> Ident.t
+  val add_fresh_ident : t -> Variable.t -> V.t * t
+  val ident_for_var_exn : t -> Variable.t -> V.t
 
-  val add_fresh_mutable_ident : t -> Mutable_variable.t -> Ident.t * t
-  val ident_for_mutable_var_exn : t -> Mutable_variable.t -> Ident.t
+  val add_fresh_mutable_ident : t -> Mutable_variable.t -> V.t * t
+  val ident_for_mutable_var_exn : t -> Mutable_variable.t -> V.t
 
   val add_allocated_const : t -> Symbol.t -> Allocated_const.t -> t
   val allocated_const_for_symbol : t -> Symbol.t -> Allocated_const.t option
@@ -134,8 +125,8 @@ module Env : sig
 end = struct
   type t =
     { subst : Clambda.ulambda Variable.Map.t;
-      var : Ident.t Variable.Map.t;
-      mutable_var : Ident.t Mutable_variable.Map.t;
+      var : V.t Variable.Map.t;
+      mutable_var : V.t Mutable_variable.Map.t;
       toplevel : bool;
       allocated_constant_for_symbol : Allocated_const.t Symbol.Map.t;
     }
@@ -156,14 +147,14 @@ end = struct
   let ident_for_var_exn t id = Variable.Map.find id t.var
 
   let add_fresh_ident t var =
-    let id = Ident.create (Variable.unique_name var) in
+    let id = V.create_local (Variable.name var) in
     id, { t with var = Variable.Map.add var id t.var }
 
   let ident_for_mutable_var_exn t mut_var =
     Mutable_variable.Map.find mut_var t.mutable_var
 
   let add_fresh_mutable_ident t mut_var =
-    let id = Mutable_variable.unique_ident mut_var in
+    let id = V.create_local (Mutable_variable.name mut_var) in
     let mutable_var = Mutable_variable.Map.add mut_var id t.mutable_var in
     id, { t with mutable_var; }
 
@@ -236,12 +227,13 @@ let rec to_clambda t env (flam : Flambda.t) : Clambda.ulambda =
   | Let { var; defining_expr; body; _ } ->
     (* TODO: synthesize proper value_kind *)
     let id, env_body = Env.add_fresh_ident env var in
-    Ulet (Immutable, Pgenval, id, to_clambda_named t env var defining_expr,
+    Ulet (Immutable, Pgenval, VP.create id,
+      to_clambda_named t env var defining_expr,
       to_clambda t env_body body)
   | Let_mutable { var = mut_var; initial_value = var; body; contents_kind } ->
     let id, env_body = Env.add_fresh_mutable_ident env mut_var in
     let def = subst_var env var in
-    Ulet (Mutable, contents_kind, id, def, to_clambda t env_body body)
+    Ulet (Mutable, contents_kind, VP.create id, def, to_clambda t env_body body)
   | Let_rec (defs, body) ->
     let env, defs =
       List.fold_right (fun (var, def) (env, defs) ->
@@ -250,7 +242,9 @@ let rec to_clambda t env (flam : Flambda.t) : Clambda.ulambda =
         defs (env, [])
     in
     let defs =
-      List.map (fun (id, var, def) -> id, to_clambda_named t env var def) defs
+      List.map (fun (id, var, def) ->
+          VP.create id, to_clambda_named t env var def)
+        defs
     in
     Uletrec (defs, to_clambda t env body)
   | Apply { func; args; kind = Direct direct_func; dbg = dbg } ->
@@ -279,7 +273,8 @@ let rec to_clambda t env (flam : Flambda.t) : Clambda.ulambda =
           us_actions_consts = const_actions;
           us_index_blocks = block_index;
           us_actions_blocks = block_actions;
-        })
+        },
+        Debuginfo.none)  (* debug info will be added by GPR#855 *)
     in
     (* Check that the [failaction] may be duplicated.  If this is not the
        case, share it through a static raise / static catch. *)
@@ -314,14 +309,15 @@ let rec to_clambda t env (flam : Flambda.t) : Clambda.ulambda =
     let env_handler, ids =
       List.fold_right (fun var (env, ids) ->
           let id, env = Env.add_fresh_ident env var in
-          env, id :: ids)
+          env, (VP.create id, Lambda.Pgenval) :: ids)
         vars (env, [])
     in
     Ucatch (Static_exception.to_int static_exn, ids,
       to_clambda t env body, to_clambda t env_handler handler)
   | Try_with (body, var, handler) ->
     let id, env_handler = Env.add_fresh_ident env var in
-    Utrywith (to_clambda t env body, id, to_clambda t env_handler handler)
+    Utrywith (to_clambda t env body, VP.create id,
+      to_clambda t env_handler handler)
   | If_then_else (arg, ifso, ifnot) ->
     Uifthenelse (subst_var env arg, to_clambda t env ifso,
       to_clambda t env ifnot)
@@ -329,7 +325,7 @@ let rec to_clambda t env (flam : Flambda.t) : Clambda.ulambda =
     Uwhile (to_clambda t env cond, to_clambda t env body)
   | For { bound_var; from_value; to_value; direction; body } ->
     let id, env_body = Env.add_fresh_ident env bound_var in
-    Ufor (id, subst_var env from_value, subst_var env to_value,
+    Ufor (VP.create id, subst_var env from_value, subst_var env to_value,
       direction, to_clambda t env_body body)
   | Assign { being_assigned; new_value } ->
     let id =
@@ -410,13 +406,28 @@ and to_clambda_switch t env cases num_keys default =
     if Numbers.Int.Set.cardinal num_keys = 0 then 0
     else Numbers.Int.Set.max_elt num_keys + 1
   in
-  let index = Array.make num_keys 0 in
   let store = Flambda_utils.Switch_storer.mk_store () in
-  begin match default with
-  | Some def when List.length cases < num_keys -> ignore (store.act_store def)
-  | _ -> ()
+  let default_action =
+    match default with
+    | Some def when List.length cases < num_keys ->
+      store.act_store () def
+    | _ -> -1
+  in
+  let index = Array.make num_keys default_action in
+  let smallest_key = ref num_keys in
+  List.iter
+    (fun (key, lam) ->
+      index.(key) <- store.act_store () lam;
+      smallest_key := min key !smallest_key
+    )
+    cases;
+  if !smallest_key < num_keys then begin
+    let action = ref index.(!smallest_key) in
+    Array.iteri
+      (fun i act ->
+         if act >= 0 then action := act else index.(i) <- !action)
+      index
   end;
-  List.iter (fun (key, lam) -> index.(key) <- store.act_store lam) cases;
   let actions = Array.map (to_clambda t env) (store.act_get ()) in
   match actions with
   | [| |] -> [| |], [| |]  (* May happen when [default] is [None]. *)
@@ -462,7 +473,7 @@ and to_clambda_set_of_closures t env
       (({ function_decls; free_vars } : Flambda.set_of_closures)
         as set_of_closures) : Clambda.ulambda =
   let all_functions = Variable.Map.bindings function_decls.funs in
-  let env_var = Ident.create "env" in
+  let env_var = V.create_local "env" in
   let to_clambda_function
         (closure_id, (function_decl : Flambda.function_declaration))
         : Clambda.ufunction =
@@ -510,13 +521,17 @@ and to_clambda_set_of_closures t env
     in
     let env_body, params =
       List.fold_right (fun var (env, params) ->
-          let id, env = Env.add_fresh_ident env var in
+          let id, env = Env.add_fresh_ident env (Parameter.var var) in
           env, id :: params)
         function_decl.params (env, [])
     in
     { label = Compilenv.function_label closure_id;
       arity = Flambda_utils.function_arity function_decl;
-      params = params @ [env_var];
+      params =
+        List.map
+          (fun var -> VP.create var, Lambda.Pgenval)
+          (params @ [env_var]);
+      return = Lambda.Pgenval;
       body = to_clambda t env_body function_decl.body;
       dbg = function_decl.dbg;
       env = Some env_var;
@@ -550,13 +565,14 @@ and to_clambda_closed_set_of_closures t env symbol
     in
     let env_body, params =
       List.fold_right (fun var (env, params) ->
-          let id, env = Env.add_fresh_ident env var in
+          let id, env = Env.add_fresh_ident env (Parameter.var var) in
           env, id :: params)
         function_decl.params (env, [])
     in
     { label = Compilenv.function_label (Closure_id.wrap id);
       arity = Flambda_utils.function_arity function_decl;
-      params;
+      params = List.map (fun var -> VP.create var, Lambda.Pgenval) params;
+      return = Lambda.Pgenval;
       body = to_clambda t env_body function_decl.body;
       dbg = function_decl.dbg;
       env = None;
@@ -568,7 +584,7 @@ and to_clambda_closed_set_of_closures t env symbol
 
 let to_clambda_initialize_symbol t env symbol fields : Clambda.ulambda =
   let fields =
-    List.mapi (fun index expr -> index, to_clambda t env expr) fields
+    List.map (fun (index, expr) -> index, to_clambda t env expr) fields
   in
   let build_setfield (index, field) : Clambda.ulambda =
     (* Note that this will never cause a write barrier hit, owing to
@@ -601,7 +617,9 @@ let accumulate_structured_constants t env symbol
 
 let to_clambda_program t env constants (program : Flambda.program) =
   let rec loop env constants (program : Flambda.program_body)
-        : Clambda.ulambda * Clambda.ustructured_constant Symbol.Map.t =
+        : Clambda.ulambda *
+          Clambda.ustructured_constant Symbol.Map.t *
+          Clambda.preallocated_block list =
     match program with
     | Let_symbol (symbol, alloc, program) ->
       (* Useful only for unboxing. Since floats and boxed integers will
@@ -623,19 +641,53 @@ let to_clambda_program t env constants (program : Flambda.program) =
           constants defs
       in
       loop env constants program
-    | Initialize_symbol (symbol, _tag, fields, program) ->
-      (* The tag is ignored here: It is used separately to generate the
-         preallocated block. Only the initialisation code is generated
-         here. *)
-      let e1 = to_clambda_initialize_symbol t env symbol fields in
-      let e2, constants = loop env constants program in
-      Usequence (e1, e2), constants
+    | Initialize_symbol (symbol, tag, fields, program) ->
+      let fields =
+        List.mapi (fun i field ->
+            i, field,
+            Initialize_symbol_to_let_symbol.constant_field field)
+          fields
+      in
+      let init_fields =
+        List.filter_map (function
+            | (i, field, None) -> Some (i, field)
+            | (_, _, Some _) -> None)
+          fields
+      in
+      let constant_fields =
+        List.map (fun (_, _, constant_field) ->
+            match constant_field with
+            | None -> None
+            | Some (Flambda.Const const) ->
+                let n =
+                  match const with
+                  | Int i -> i
+                  | Char c -> Char.code c
+                  | Const_pointer i -> i
+                in
+                Some (Clambda.Uconst_field_int n)
+            | Some (Flambda.Symbol sym) ->
+                let lbl = Linkage_name.to_string (Symbol.label sym) in
+                Some (Clambda.Uconst_field_ref lbl))
+          fields
+      in
+      let e1 = to_clambda_initialize_symbol t env symbol init_fields in
+      let preallocated_block : Clambda.preallocated_block =
+        { symbol = Linkage_name.to_string (Symbol.label symbol);
+          exported = true;
+          tag = Tag.to_int tag;
+          fields = constant_fields;
+          provenance = None;
+        }
+      in
+      let e2, constants, preallocated_blocks = loop env constants program in
+      Usequence (e1, e2), constants, preallocated_block :: preallocated_blocks
     | Effect (expr, program) ->
       let e1 = to_clambda t env expr in
-      let e2, constants = loop env constants program in
-      Usequence (e1, e2), constants
+      let e2, constants, preallocated_blocks = loop env constants program in
+      Usequence (e1, e2), constants, preallocated_blocks
     | End _ ->
-      Uconst (Uconst_ptr 0), constants
+      Uconst (Uconst_ptr 0), constants, []
   in
   loop env constants program.program_body
 
@@ -646,49 +698,52 @@ type result = {
   exported : Export_info.t;
 }
 
-let convert (program, exported) : result =
+let convert (program, exported_transient) : result =
   let current_unit =
+    let closures =
+      Closure_id.Map.keys (Flambda_utils.make_closure_map program)
+    in
+    let constant_closures =
+      Flambda_utils.all_lifted_constant_closures program
+    in
     let offsets = Closure_offsets.compute program in
     { fun_offset_table = offsets.function_offsets;
       fv_offset_table = offsets.free_variable_offsets;
-      closures = Flambda_utils.make_closure_map program;
-      constant_sets_of_closures =
-        Flambda_utils.all_lifted_constant_sets_of_closures program;
+      constant_closures;
+      closures;
     }
   in
   let imported_units =
     let imported = Compilenv.approx_env () in
+    let closures =
+      Set_of_closures_id.Map.fold
+        (fun (_ : Set_of_closures_id.t) fun_decls acc ->
+           Variable.Map.fold
+             (fun var (_ : Simple_value_approx.function_declaration) acc ->
+               let closure_id = Closure_id.wrap var in
+               Closure_id.Set.add closure_id acc)
+             fun_decls.Simple_value_approx.funs
+             acc)
+        imported.sets_of_closures
+        Closure_id.Set.empty
+    in
     { fun_offset_table = imported.offset_fun;
       fv_offset_table = imported.offset_fv;
-      closures = imported.closures;
-      constant_sets_of_closures = imported.constant_sets_of_closures;
+      constant_closures = imported.constant_closures;
+      closures;
     }
   in
   let t = { current_unit; imported_units; } in
-  let preallocated_blocks =
-    List.map (fun (symbol, tag, fields) ->
-        { Clambda.
-          symbol = Linkage_name.to_string (Symbol.label symbol);
-          exported = true;
-          tag = Tag.to_int tag;
-          size = List.length fields;
-        })
-      (Flambda_utils.initialize_symbols program)
-  in
-  let expr, structured_constants =
+  let expr, structured_constants, preallocated_blocks =
     to_clambda_program t Env.empty Symbol.Map.empty program
   in
-  let offset_fun, offset_fv =
-    Closure_offsets.compute_reexported_offsets program
-      ~current_unit_offset_fun:current_unit.fun_offset_table
-      ~current_unit_offset_fv:current_unit.fv_offset_table
-      ~imported_units_offset_fun:imported_units.fun_offset_table
-      ~imported_units_offset_fv:imported_units.fv_offset_table
-  in
   let exported =
-    Export_info.add_clambda_info exported
-      ~offset_fun
-      ~offset_fv
-      ~constant_sets_of_closures:current_unit.constant_sets_of_closures
+    Export_info.t_of_transient exported_transient
+      ~program
+      ~local_offset_fun:current_unit.fun_offset_table
+      ~local_offset_fv:current_unit.fv_offset_table
+      ~imported_offset_fun:imported_units.fun_offset_table
+      ~imported_offset_fv:imported_units.fv_offset_table
+      ~constant_closures:current_unit.constant_closures
   in
   { expr; preallocated_blocks; structured_constants; exported; }
