@@ -15,11 +15,10 @@
 
 (* Link .cmo files and produce a bytecode executable. *)
 
-val link : Format.formatter -> string list -> string -> unit
+val link : string list -> string -> unit
 val reset : unit -> unit
 
-val check_consistency:
-  Format.formatter -> string -> Cmo_format.compilation_unit -> unit
+val check_consistency: string -> Cmo_format.compilation_unit -> unit
 
 val extract_crc_interfaces: unit -> (string * Digest.t option) list
 
@@ -32,7 +31,6 @@ type error =
   | Custom_runtime
   | File_exists of string
   | Cannot_open_dll of string
-  | Not_compatible_32
   | Required_module_unavailable of string
 
 exception Error of error
