@@ -19,18 +19,18 @@
 #include "m.h"
 
 #if defined(ARCH_INT64_TYPE)
-typedef ARCH_INT64_TYPE int64_t;
+typedef ARCH_INT64_TYPE myint64_t;
 #elif SIZEOF_LONG == 8
-typedef long int64_t;
+typedef long myint64_t;
 #elif SIZEOF_LONGLONG == 8
-typedef long long int64_t;
+typedef long long myint64_t;
 #else
 #error "No 64-bit integer type available"
 #endif
 
-volatile int64_t foo;
+volatile myint64_t foo;
 
-void access_int64(volatile int64_t *p)
+void access_int64(volatile myint64_t *p)
 {
   foo = *p;
 }
@@ -51,8 +51,8 @@ int main(void)
   signal(SIGBUS, sig_handler);
 #endif
   if(setjmp(failure) == 0) {
-    access_int64((volatile int64_t *) n);
-    access_int64((volatile int64_t *) (n+1));
+    access_int64((volatile myint64_t *) n);
+    access_int64((volatile myint64_t *) (n+1));
     res = 0;
   } else {
     res = 1;
