@@ -19,8 +19,7 @@
 open Instruct
 open Debugger_config (* Toplevel *)
 open Program_loading
-
-module StringSet = Set.Make(String)
+module String = Misc.Stdlib.String
 
 let modules =
   ref ([] : string list)
@@ -37,6 +36,18 @@ let events_by_module =
 let all_events_by_module =
   (Hashtbl.create 17 : (string, debug_event list) Hashtbl.t)
 
+let partition_modules evl =
+  let rec partition_modules' ev evl =
+    match evl with
+      [] -> [ev],[]
+    | ev'::evl ->
+       let evl,evll = partition_modules' ev' evl in
+       if ev.ev_module = ev'.ev_module then ev::evl,evll else [ev],evl::evll
+  in
+  match evl with
+    [] -> []
+  | ev::evl -> let evl,evll = partition_modules' ev evl in evl::evll
+
 let relocate_event orig ev =
   ev.ev_pos <- orig + ev.ev_pos;
   match ev.ev_repr with
@@ -60,16 +71,17 @@ let read_symbols' bytecode_file =
     raise Toplevel
   end;
   let num_eventlists = input_binary_int ic in
-  let dirs = ref StringSet.empty in
+  let dirs = ref String.Set.empty in
   let eventlists = ref [] in
   for _i = 1 to num_eventlists do
     let orig = input_binary_int ic in
     let evl = (input_value ic : debug_event list) in
     (* Relocate events in event list *)
     List.iter (relocate_event orig) evl;
-    eventlists := evl :: !eventlists;
+    let evll = partition_modules evl in
+    eventlists := evll @ !eventlists;
     dirs :=
-      List.fold_left (fun s e -> StringSet.add e s) !dirs (input_value ic)
+      List.fold_left (fun s e -> String.Set.add e s) !dirs (input_value ic)
   done;
   begin try
     ignore (Bytesections.seek_section ic "CODE")
@@ -85,7 +97,7 @@ let read_symbols bytecode_file =
   let all_events, all_dirs = read_symbols' bytecode_file in
 
   modules := []; events := [];
-  program_source_dirs := StringSet.elements all_dirs;
+  program_source_dirs := String.Set.elements all_dirs;
   Hashtbl.clear events_by_pc; Hashtbl.clear events_by_module;
   Hashtbl.clear all_events_by_module;
 
@@ -154,7 +166,10 @@ let find_event ev char =
       else bsearch (pivot + 1) hi
     end
   in
-  bsearch 0 (Array.length ev - 1)
+  if Array.length ev = 0 then
+    raise Not_found
+  else
+    bsearch 0 (Array.length ev - 1)
 
 (* Return first event after the given position. *)
 (* Raise [Not_found] if module is unknown or no event is found. *)
