@@ -15,7 +15,7 @@
 
 open Compenv
 open Parsetree
-module StringMap = Depend.StringMap
+module String = Misc.Stdlib.String
 
 let ppf = Format.err_formatter
 (* Print the dependencies *)
@@ -34,10 +34,10 @@ let sort_files = ref false
 let all_dependencies = ref false
 let one_line = ref false
 let files =
-  ref ([] : (string * file_kind * Depend.StringSet.t * string list) list)
+  ref ([] : (string * file_kind * String.Set.t * string list) list)
 let allow_approximation = ref false
 let map_files = ref []
-let module_map = ref StringMap.empty
+let module_map = ref String.Map.empty
 let debug = ref false
 
 (* Fix path to use '/' as directory separator instead of '\'.
@@ -50,10 +50,10 @@ let fix_slash s =
 
 (* Since we reinitialize load_path after reading OCAMLCOMP,
   we must use a cache instead of calling Sys.readdir too often. *)
-let dirs = ref StringMap.empty
+let dirs = ref String.Map.empty
 let readdir dir =
   try
-    StringMap.find dir !dirs
+    String.Map.find dir !dirs
   with Not_found ->
     let contents =
       try
@@ -63,7 +63,7 @@ let readdir dir =
         error_occurred := true;
         [||]
     in
-    dirs := StringMap.add dir contents !dirs;
+    dirs := String.Map.add dir contents !dirs;
     contents
 
 let add_to_list li s =
@@ -189,21 +189,36 @@ let print_filename s =
 ;;
 
 let print_dependencies target_files deps =
-  let rec print_items pos = function
-    [] -> print_string "\n"
-  | dep :: rem ->
-    if !one_line || (pos + 1 + String.length dep <= 77) then begin
-        if pos <> 0 then print_string " "; print_filename dep;
-        print_items (pos + String.length dep + 1) rem
-      end else begin
-        print_string escaped_eol; print_filename dep;
-        print_items (String.length dep + 4) rem
-      end in
-  print_items 0 (target_files @ [depends_on] @ deps)
+  let pos = ref 0 in
+  let print_on_same_line item =
+    if !pos <> 0 then print_string " ";
+    print_filename item;
+    pos := !pos + String.length item + 1;
+  in
+  let print_on_new_line item =
+    print_string escaped_eol;
+    print_filename item;
+    pos := String.length item + 4;
+  in
+  let print_compact item =
+    if !one_line || (!pos + 1 + String.length item <= 77)
+    then print_on_same_line item
+    else print_on_new_line item
+  in
+  let print_dep item =
+    if !one_line
+    then print_on_same_line item
+    else print_on_new_line item
+  in
+  List.iter print_compact target_files;
+  print_string " "; print_string depends_on;
+  pos := !pos + String.length depends_on + 1;
+  List.iter print_dep deps;
+  print_string "\n"
 
 let print_raw_dependencies source_file deps =
   print_filename source_file; print_string depends_on;
-  Depend.StringSet.iter
+  String.Set.iter
     (fun dep ->
        (* filter out "*predef*" *)
       if (String.length dep > 0)
@@ -240,7 +255,7 @@ let rec lexical_approximation lexbuf =
       match Lexer.token lexbuf with
       | Parser.UIDENT name ->
           Depend.free_structure_names :=
-            Depend.StringSet.add name !Depend.free_structure_names;
+            String.Set.add name !Depend.free_structure_names;
           process false lexbuf
       | Parser.LIDENT _ -> process true lexbuf
       | Parser.DOT when after_lident -> process false lexbuf
@@ -259,7 +274,7 @@ let rec lexical_approximation lexbuf =
 
 let read_and_approximate inputfile =
   error_occurred := false;
-  Depend.free_structure_names := Depend.StringSet.empty;
+  Depend.free_structure_names := String.Set.empty;
   let ic = open_in_bin inputfile in
   try
     seek_in ic 0;
@@ -277,14 +292,11 @@ let read_and_approximate inputfile =
 let read_parse_and_extract parse_function extract_function def ast_kind
     source_file =
   Depend.pp_deps := [];
-  Depend.free_structure_names := Depend.StringSet.empty;
+  Depend.free_structure_names := String.Set.empty;
   try
     let input_file = Pparse.preprocess source_file in
     begin try
-      let ast =
-        Pparse.file ~tool_name Format.err_formatter
-                    input_file parse_function ast_kind
-      in
+      let ast = Pparse.file ~tool_name input_file parse_function ast_kind in
       let bound_vars =
         List.fold_left
           (fun bv modname ->
@@ -301,7 +313,7 @@ let read_parse_and_extract parse_function extract_function def ast_kind
   with x -> begin
     report_err x;
     if not !allow_approximation
-    then (Depend.StringSet.empty, def)
+    then (String.Set.empty, def)
     else (read_and_approximate source_file, def)
   end
 
@@ -323,7 +335,7 @@ let print_ml_dependencies source_file extracted_deps pp_deps =
          (if !all_dependencies then [cmi_name] else [])
   in
   let (byt_deps, native_deps) =
-    Depend.StringSet.fold (find_dependency ML)
+    String.Set.fold (find_dependency ML)
       extracted_deps init_deps in
   if not !native_only then
     print_dependencies (byte_targets @ extra_targets) (byt_deps @ pp_deps);
@@ -339,7 +351,7 @@ let print_ml_dependencies source_file extracted_deps pp_deps =
 let print_mli_dependencies source_file extracted_deps pp_deps =
   let basename = Filename.chop_extension source_file in
   let (byt_deps, _opt_deps) =
-    Depend.StringSet.fold (find_dependency MLI)
+    String.Set.fold (find_dependency MLI)
       extracted_deps ([], []) in
   print_dependencies [basename ^ ".cmi"] (byt_deps @ pp_deps)
 
@@ -425,7 +437,7 @@ let sort_files_by_dependencies files =
     let add_dep modname kind =
       new_deps := (modname, kind) :: !new_deps;
     in
-    Depend.StringSet.iter (fun modname ->
+    String.Set.iter (fun modname ->
       match file_kind with
           ML -> (* ML depends both on ML and MLI *)
             if Hashtbl.mem h (modname, MLI) then add_dep modname MLI;
@@ -463,8 +475,8 @@ let sort_files_by_dependencies files =
   done;
 
   if !worklist <> [] then begin
-    Format.fprintf Format.err_formatter
-      "@[Warning: cycle in dependencies. End of list is not sorted.@]@.";
+    Location.error "cycle in dependencies. End of list is not sorted."
+    |> Location.print_report Format.err_formatter;
     let sorted_deps =
       let li = ref [] in
       Hashtbl.iter (fun _ file_deps -> li := file_deps :: !li) h;
@@ -478,6 +490,7 @@ let sort_files_by_dependencies files =
       ) !deps;
       Format.fprintf Format.err_formatter "@]@.";
       Printf.printf "%s " file) sorted_deps;
+    error_occurred := true
   end;
   Printf.printf "\n%!";
   ()
@@ -486,30 +499,30 @@ let sort_files_by_dependencies files =
 
 let rec dump_map s0 ppf m =
   let open Depend in
-  StringMap.iter
+  String.Map.iter
     (fun key (Node(s1,m')) ->
-      let s = StringSet.diff s1 s0 in
-      if StringSet.is_empty s then
+      let s = String.Set.diff s1 s0 in
+      if String.Set.is_empty s then
         Format.fprintf ppf "@ @[<hv2>module %s : sig%a@;<1 -2>end@]"
-          key (dump_map (StringSet.union s1 s0)) m'
+          key (dump_map (String.Set.union s1 s0)) m'
       else
-        Format.fprintf ppf "@ module %s = %s" key (StringSet.choose s))
+        Format.fprintf ppf "@ module %s = %s" key (String.Set.choose s))
     m
 
 let process_ml_map =
   read_parse_and_extract Parse.implementation Depend.add_implementation_binding
-                         StringMap.empty Pparse.Structure
+                         String.Map.empty Pparse.Structure
 
 let process_mli_map =
   read_parse_and_extract Parse.interface Depend.add_signature_binding
-                         StringMap.empty Pparse.Signature
+                         String.Map.empty Pparse.Signature
 
 let parse_map fname =
   map_files := fname :: !map_files ;
   let old_transp = !Clflags.transparent_modules in
   Clflags.transparent_modules := true;
   let (deps, m) =
-    process_file fname ~def:(Depend.StringSet.empty, StringMap.empty)
+    process_file fname ~def:(String.Set.empty, String.Map.empty)
       ~ml_file:process_ml_map
       ~mli_file:process_mli_map
   in
@@ -517,16 +530,16 @@ let parse_map fname =
   let modname =
     String.capitalize_ascii
       (Filename.basename (Filename.chop_extension fname)) in
-  if StringMap.is_empty m then
+  if String.Map.is_empty m then
     report_err (Failure (fname ^ " : empty map file or parse error"));
   let mm = Depend.make_node m in
   if !debug then begin
     Format.printf "@[<v>%s:%t%a@]@." fname
-      (fun ppf -> Depend.StringSet.iter (Format.fprintf ppf " %s") deps)
-      (dump_map deps) (StringMap.add modname mm StringMap.empty)
+      (fun ppf -> String.Set.iter (Format.fprintf ppf " %s") deps)
+      (dump_map deps) (String.Map.add modname mm String.Map.empty)
   end;
-  let mm = Depend.(weaken_map (StringSet.singleton modname) mm) in
-  module_map := StringMap.add modname mm !module_map
+  let mm = Depend.(weaken_map (String.Set.singleton modname) mm) in
+  module_map := String.Map.add modname mm !module_map
 ;;
 
 
@@ -548,7 +561,7 @@ let main () =
   Compenv.readenv ppf Before_args;
   Clflags.reset_arguments (); (* reset arguments from ocamlc/ocamlopt *)
   Clflags.add_arguments __LOC__ [
-     "-absname", Arg.Set Location.absname,
+     "-absname", Arg.Set Clflags.absname,
         " Show absolute filenames in error messages";
      "-all", Arg.Set all_dependencies,
         " Generate dependencies on all files";
