@@ -285,6 +285,25 @@ not empty or "dumb", and that isatty(stderr) holds.
 The environment variable "OCAML_COLOR" is considered if \-color is not
 provided. Its values are auto/always/never as above.
 
+.TP
+.BI \-error\-style \ mode
+Control the way error messages and warnings are printed.
+The following modes are supported:
+
+.B short
+only print the error and its location;
+
+.B contextual
+like "short", but also display the source code snippet corresponding
+to the location of the error.
+
+The default setting is
+.B contextual.
+
+The environment variable "OCAML_ERROR_STYLE" is considered if
+\-error\-style is not provided. Its values are short/contextual as
+above.
+
 .TP
 .B \-compat\-32
 Check that the generated bytecode executable can run on 32-bit
@@ -296,6 +315,13 @@ Print the version number of
 .BR ocamlc (1)
 and a detailed summary of its configuration, then exit.
 .TP
+.BI \-config-var
+Print the value of a specific configuration variable
+from the
+.B \-config
+output, then exit. If the variable does not exist,
+the exit code is non-zero.
+.TP
 .B \-custom
 Link in "custom runtime" mode. In the default linking mode, the
 linker produces bytecode that is intended to be executed with the
@@ -452,6 +478,11 @@ bytecode executables produced with the option
 .B ocamlc\ \-use\-runtime
 .IR runtime-name .
 .TP
+.B \-match\-context\-rows
+Set number of rows of context used during pattern matching
+compilation. Lower values cause faster compilation, but
+less optimized code. The default value is 32.
+.TP
 .B \-no-alias-deps
 Do not record dependencies for module aliases.
 .TP
@@ -608,11 +639,16 @@ then the
 .B d
 suffix is supported and gives a debug version of the runtime.
 .TP
+.BI \-stop\-after \ pass
+Stop compilation after the given compilation pass. The currently
+supported passes are:
+.BR parsing ,
+.BR typing .
+.TP
 .B \-safe\-string
 Enforce the separation between types
 .BR string \ and\  bytes ,
-thereby making strings read-only. This will become the default in
-a future version of OCaml.
+thereby making strings read-only. This is the default.
 .TP
 .B \-short\-paths
 When a type is visible under several module-paths, use the shortest
@@ -622,11 +658,6 @@ warning messages.
 .B \-strict\-sequence
 Force the left-hand part of each sequence to have type unit.
 .TP
-.B \-thread
-Compile or link multithreaded programs, in combination with the
-system "threads" library described in
-.IR The\ OCaml\ user's\ manual .
-.TP
 .B \-unboxed\-types
 When a type is unboxable (i.e. a record with a single argument or a
 concrete datatype with a single constructor of one argument) it will
@@ -650,9 +681,9 @@ accesses an array or string outside of its bounds.
 .B \-unsafe\-string
 Identify the types
 .BR string \ and\  bytes ,
-thereby making strings writable. For reasons of backward compatibility,
-this is the default setting for the moment, but this will change in a future
-version of OCaml.
+thereby making strings writable.
+This is intended for compatibility with old source code and should not
+be used with new software.
 .TP
 .BI \-use\-runtime \ runtime\-name
 Generate a bytecode executable file that can be executed on the custom
@@ -673,8 +704,8 @@ invocations of the C compiler and linker in
 mode.  Useful to debug C library problems.
 .TP
 .B \-vmthread
-Compile or link multithreaded programs, in combination with the
-VM-level threads library described in
+Deprecated since OCaml 4.08.0. Compile or link multithreaded programs,
+in combination with the VM-level threads library described in
 .IR The\ OCaml\ user's\ manual .
 .TP
 .BR \-vnum \ or\  \-version
@@ -933,6 +964,21 @@ mutually recursive types.
 61
 \ \ Unannotated unboxable type in primitive declaration.
 
+62
+\ \ Type constraint on GADT type declaration
+
+63
+\ \ Erroneous printed signature
+
+64
+\ \ -unsafe used with a preprocessor returning a syntax tree
+
+65
+\ \ Type declaration defining a new '()' constructor
+
+66
+\ \ Unused open! statement.
+
 The letters stand for the following sets of warnings.  Any letter not
 mentioned here corresponds to the empty set.
 
@@ -986,7 +1032,7 @@ mentioned here corresponds to the empty set.
 
 .IP
 The default setting is
-.BR \-w\ +a\-4\-6\-7\-9\-27\-29\-32..42\-44\-45\-48\-50\-60 .
+.BR \-w\ +a\-4\-6\-7\-9\-27\-29\-32..42\-44\-45\-48\-50\-60\-66 .
 Note that warnings
 .BR 5 \ and \ 10
 are not always triggered, depending on the internals of the type checker.
