@@ -158,17 +158,17 @@ following files.
 .BI \-plugin \ plugin
 Dynamically load the code of the given
 .I plugin
-(a .cmo, .cma or .cmxs file) in 
+(a .cmo, .cma or .cmxs file) in
 .BR ocamldep (1).
 The plugin must exist in
 the same kind of code as the tool (
-.BR ocamldep.byte 
+.BR ocamldep.byte
 must load bytecode
-plugins, while 
+plugins, while
 .BR ocamldep.opt
 must load native code plugins), and
 extension adaptation is done automatically for .cma files (to .cmxs files
-if 
+if
 .BR ocamldep (1)
 is compiled in native code).
 .TP
@@ -183,6 +183,10 @@ as a preprocessor for each source file.
 Pipe abstract syntax tree through preprocessor
 .IR command .
 .TP
+.B \-shared
+Generate dependencies for native plugin files (.cmxs) in addition to
+native compiled files (.cmx).
+.TP
 .B \-slash
 Under Unix, this option does nothing.
 .TP
