@@ -108,9 +108,9 @@ let no_header = " Suppress header in generated documentation\n\t\t"^latex_texi_o
 let no_trailer = " Suppress trailer in generated documentation\n\t\t"^latex_texi_only
 let separate_files = " Generate one file per toplevel module "^latex_only
 let latex_title ref_titles =
-  "n,style Associate {n } to the given sectionning style\n"^
+  "n,style Associate {n } to the given sectioning style\n"^
   "\t\t(e.g. 'section') in the latex output "^latex_only^"\n"^
-  "\t\tDefault sectionning is:\n\t\t"^
+  "\t\tDefault sectioning is:\n\t\t"^
   (String.concat "\n\t\t"
      (List.map (fun (n,t) -> Printf.sprintf " %d -> %s" n t) !ref_titles))
 
@@ -204,9 +204,9 @@ let merge_all = ('A', "merge all")
 let no_index = " Do not build index for Info files "^texi_only
 let esc_8bits = " Escape accentuated characters in Info files "^texi_only
 let texinfo_title r=
-  "n,style Associate {n } to the given sectionning style\n"^
+  "n,style Associate {n } to the given sectioning style\n"^
   "\t\t(e.g. 'section') in the texInfo output "^texi_only^"\n"^
-  "\t\tDefault sectionning is:\n\t\t"^
+  "\t\tDefault sectioning is:\n\t\t"^
   (String.concat "\n\t\t"
      (List.map (fun (n,(t,h)) ->
           Printf.sprintf " %d -> %s, %s " n t h) !r))
@@ -238,6 +238,13 @@ let merge_options =
        merge_all ]
   )
 
+let initially_opened_module = "<module> Name of the module that is initially opened"
+
+let library_namespace =
+  "<module> Name of the library namespace for a prefixed library.\
+   Note: very experimental."
+
+
 let help = " Display this list of options"
 
 
@@ -255,7 +262,7 @@ let bad_magic_number =
 let not_a_module_name s = s^" is not a valid module name"
 let load_file_error f e = "Error while loading file "^f^":\n"^e
 let wrong_format s = "Wrong format for \""^s^"\""
-let errors_occured n = (string_of_int n)^" error(s) encountered"
+let errors_occured n = (Int.to_string n)^" error(s) encountered"
 let parse_error = "Parse error"
 let text_parse_error l c s =
   let lines = Str.split (Str.regexp_string "\n") s in
@@ -309,7 +316,7 @@ let module_not_found_in_typedtree m = "Module "^m^" was not found in typed tree.
 let class_not_found_in_typedtree c = "Class "^c^" was not found in typed tree."
 let class_type_not_found_in_typedtree ct = "Class type "^ct^" was not found in typed tree."
 let inherit_classexp_not_found_in_typedtree n =
-  "Inheritance class expression number "^(string_of_int n)^" was not found in typed tree."
+  "Inheritance class expression number "^(Int.to_string n)^" was not found in typed tree."
 let attribute_not_found_in_typedtree att = "Class attribute "^att^" was not found in typed tree."
 let method_not_found_in_typedtree met = "Class method "^met^" was not found in typed tree."
 let misplaced_comment file pos =
