@@ -116,7 +116,7 @@ let rec string_of_text t =
           let rec f n = function
               [] -> "\n"
             | t :: q ->
-                "\n"^(string_of_int n)^". "^(string_of_text t)^
+                "\n"^(Int.to_string n)^". "^(string_of_text t)^
                 (f (n + 1) q)
           in
           f 1 l
@@ -229,14 +229,14 @@ let apply_opt f v_opt =
 let string_of_date ?(absolute=false) ?(hour=true) d =
   let add_0 s = if String.length s < 2 then "0"^s else s in
   let t = (if absolute then Unix.gmtime else Unix.localtime) d in
-  (string_of_int (t.Unix.tm_year + 1900))^"-"^
-  (add_0 (string_of_int (t.Unix.tm_mon + 1)))^"-"^
-  (add_0 (string_of_int t.Unix.tm_mday))^
+  (Int.to_string (t.Unix.tm_year + 1900))^"-"^
+  (add_0 (Int.to_string (t.Unix.tm_mon + 1)))^"-"^
+  (add_0 (Int.to_string t.Unix.tm_mday))^
   (
    if hour then
      " "^
-     (add_0 (string_of_int t.Unix.tm_hour))^":"^
-     (add_0 (string_of_int t.Unix.tm_min))
+     (add_0 (Int.to_string t.Unix.tm_hour))^":"^
+     (add_0 (Int.to_string t.Unix.tm_min))
    else
      ""
   )
