@@ -31,7 +31,8 @@ static struct custom_operations image_ops = {
   custom_hash_default,
   custom_serialize_default,
   custom_deserialize_default,
-  custom_compare_ext_default
+  custom_compare_ext_default,
+  custom_fixed_length_default
 };
 
 #define Max_image_mem 2000000
