@@ -63,6 +63,8 @@ let _ = thread_initialize()
 
 (* Back to the Unix module *)
 
+let shell = "/bin/sh"
+
 type error =
     E2BIG
   | EACCES
@@ -159,7 +161,9 @@ let handle_unix_error f arg =
     exit 2
 
 external environment : unit -> string array = "unix_environment"
+external unsafe_environment : unit -> string array = "unix_environment_unsafe"
 external getenv: string -> string = "caml_sys_getenv"
+external unsafe_getenv: string -> string = "caml_sys_unsafe_getenv"
 external putenv: string -> string -> unit = "unix_putenv"
 
 type interval_timer =
@@ -208,6 +212,7 @@ external openfile : string -> open_flag list -> file_perm -> file_descr
            = "unix_open"
 
 external close : file_descr -> unit = "unix_close"
+external fsync : file_descr -> unit = "unix_fsync"
 external unsafe_read : file_descr -> bytes -> int -> int -> int = "unix_read"
 external unsafe_write : file_descr -> bytes -> int -> int -> int
     = "unix_write"
@@ -291,7 +296,7 @@ external fstat : file_descr -> stats = "unix_fstat"
 external isatty : file_descr -> bool = "unix_isatty"
 external unlink : string -> unit = "unix_unlink"
 external rename : string -> string -> unit = "unix_rename"
-external link : string -> string -> unit = "unix_link"
+external link : ?follow:bool -> string -> string -> unit = "unix_link"
 
 module LargeFile =
   struct
@@ -318,6 +323,16 @@ module LargeFile =
     external fstat : file_descr -> stats = "unix_fstat_64"
   end
 
+external map_internal:
+   file_descr -> ('a, 'b) Stdlib.Bigarray.kind
+              -> 'c Stdlib.Bigarray.layout
+              -> bool -> int array -> int64
+              -> ('a, 'b, 'c) Stdlib.Bigarray.Genarray.t
+     = "caml_unix_map_file_bytecode" "caml_unix_map_file"
+
+let map_file fd ?(pos=0L) kind layout shared dims =
+  map_internal fd kind layout shared dims pos
+
 type access_permission =
     R_OK
   | W_OK
@@ -561,7 +576,7 @@ type msg_flag =
   | MSG_DONTROUTE
   | MSG_PEEK
 
-external _socket : 
+external _socket :
   ?cloexec: bool -> socket_domain -> socket_type -> int -> file_descr
   = "unix_socket"
 external _socketpair :
@@ -872,7 +887,7 @@ let getnameinfo_emulation addr opts =
           let kind = if List.mem NI_DGRAM opts then "udp" else "tcp" in
           (getservbyport p kind).s_name
         with Not_found ->
-          string_of_int p in
+          Int.to_string p in
       { ni_hostname = hostname; ni_service = service }
 
 let getnameinfo addr opts =
@@ -950,7 +965,7 @@ let rec waitpid_non_intr pid =
 let system cmd =
   match fork() with
      0 -> begin try
-            execv "/bin/sh" [| "/bin/sh"; "-c"; cmd |]
+            execv shell [| shell; "-c"; cmd |]
           with _ ->
             exit 127
           end
@@ -1004,25 +1019,23 @@ type popen_process =
 
 let popen_processes = (Hashtbl.create 7 : (popen_process, int) Hashtbl.t)
 
-let open_proc cmd envopt proc input output error =
+let open_proc prog args envopt proc input output error =
   match fork() with
      0 -> begin try
             perform_redirections input output error;
-            let shell = "/bin/sh" in
-            let argv = [| shell; "-c"; cmd |] in
             match envopt with
-            | Some env -> execve shell argv env
-            | None     -> execv shell argv
+            | Some env -> execve prog args env
+            | None     -> execv prog args
           with _ ->
             exit 127
           end
   | id -> Hashtbl.add popen_processes proc id
 
-let open_process_in cmd =
+let open_process_args_in prog args =
   let (in_read, in_write) = pipe ~cloexec:true () in
   let inchan = in_channel_of_descr in_read in
   try
-    open_proc cmd None (Process_in inchan) stdin in_write stderr;
+    open_proc prog args None (Process_in inchan) stdin in_write stderr;
     close in_write;
     inchan
   with e ->
@@ -1030,11 +1043,11 @@ let open_process_in cmd =
     close in_write;
     raise e
 
-let open_process_out cmd =
+let open_process_args_out prog args =
   let (out_read, out_write) = pipe ~cloexec:true () in
   let outchan = out_channel_of_descr out_write in
   try
-    open_proc cmd None (Process_out outchan) out_read stdout stderr;
+    open_proc prog args None (Process_out outchan) out_read stdout stderr;
     close out_read;
     outchan
   with e ->
@@ -1042,14 +1055,15 @@ let open_process_out cmd =
     close out_read;
     raise e
 
-let open_process cmd =
+let open_process_args prog args =
   let (in_read, in_write) = pipe ~cloexec:true () in
   let inchan = in_channel_of_descr in_read in
   try
     let (out_read, out_write) = pipe ~cloexec:true () in
     let outchan = out_channel_of_descr out_write in
     try
-      open_proc cmd None (Process(inchan, outchan)) out_read in_write stderr;
+      open_proc prog args None
+                (Process(inchan, outchan)) out_read in_write stderr;
       close out_read;
       close in_write;
       (inchan, outchan)
@@ -1060,9 +1074,9 @@ let open_process cmd =
   with e ->
     close_in inchan;
     close in_write;
-    raise e    
+    raise e
 
-let open_process_full cmd env =
+let open_process_args_full prog args env =
   let (in_read, in_write) = pipe ~cloexec:true () in
   let inchan = in_channel_of_descr in_read in
   try
@@ -1072,7 +1086,7 @@ let open_process_full cmd env =
       let (err_read, err_write) = pipe ~cloexec:true () in
       let errchan = in_channel_of_descr err_read in
       try
-        open_proc cmd (Some env) (Process_full(inchan, outchan, errchan))
+        open_proc prog args (Some env) (Process_full(inchan, outchan, errchan))
                   out_read in_write err_write;
         close out_read;
         close in_write;
@@ -1089,75 +1103,66 @@ let open_process_full cmd env =
   with e ->
     close_in inchan;
     close in_write;
-    raise e    
+    raise e
+
+let open_process_shell fn cmd =
+  fn shell [|shell; "-c"; cmd|]
+let open_process_in cmd =
+  open_process_shell open_process_args_in cmd
+let open_process_out cmd =
+  open_process_shell open_process_args_out cmd
+let open_process cmd =
+  open_process_shell open_process_args cmd
+let open_process_full cmd =
+  open_process_shell open_process_args_full cmd
 
 let find_proc_id fun_name proc =
   try
-    let pid = Hashtbl.find popen_processes proc in
-    Hashtbl.remove popen_processes proc;
-    pid
+    Hashtbl.find popen_processes proc
   with Not_found ->
     raise(Unix_error(EBADF, fun_name, ""))
 
+let remove_proc_id proc =
+  Hashtbl.remove popen_processes proc
+
+let process_in_pid inchan =
+  find_proc_id "process_in_pid" (Process_in inchan)
+let process_out_pid outchan =
+  find_proc_id "process_out_pid" (Process_out outchan)
+let process_pid (inchan, outchan) =
+  find_proc_id "process_pid" (Process(inchan, outchan))
+let process_full_pid (inchan, outchan, errchan) =
+  find_proc_id "process_full_pid"
+    (Process_full(inchan, outchan, errchan))
+
 let close_process_in inchan =
-  let pid = find_proc_id "close_process_in" (Process_in inchan) in
+  let proc = Process_in inchan in
+  let pid = find_proc_id "close_process_in" proc in
+  remove_proc_id proc;
   close_in inchan;
   snd(waitpid_non_intr pid)
 
 let close_process_out outchan =
-  let pid = find_proc_id "close_process_out" (Process_out outchan) in
+  let proc = Process_out outchan in
+  let pid = find_proc_id "close_process_out" proc in
+  remove_proc_id proc;
   (* The application may have closed [outchan] already to signal
      end-of-input to the process.  *)
   begin try close_out outchan with Sys_error _ -> () end;
   snd(waitpid_non_intr pid)
 
 let close_process (inchan, outchan) =
-  let pid = find_proc_id "close_process" (Process(inchan, outchan)) in
+  let proc = Process(inchan, outchan) in
+  let pid = find_proc_id "close_process" proc in
+  remove_proc_id proc;
   close_in inchan;
   begin try close_out outchan with Sys_error _ -> () end;
   snd(waitpid_non_intr pid)
 
 let close_process_full (inchan, outchan, errchan) =
-  let pid =
-    find_proc_id "close_process_full"
-                 (Process_full(inchan, outchan, errchan)) in
-  close_in inchan;
-  begin try close_out outchan with Sys_error _ -> () end;
-  close_in errchan;
-  snd(waitpid_non_intr pid)
-
-let find_proc_id fun_name proc =
-  try
-    let pid = Hashtbl.find popen_processes proc in
-    Hashtbl.remove popen_processes proc;
-    pid
-  with Not_found ->
-    raise(Unix_error(EBADF, fun_name, ""))
-
-let rec waitpid_non_intr pid =
-  try waitpid [] pid
-  with Unix_error (EINTR, _, _) -> waitpid_non_intr pid
-
-let close_process_in inchan =
-  let pid = find_proc_id "close_process_in" (Process_in inchan) in
-  close_in inchan;
-  snd(waitpid_non_intr pid)
-
-let close_process_out outchan =
-  let pid = find_proc_id "close_process_out" (Process_out outchan) in
-  close_out outchan;
-  snd(waitpid_non_intr pid)
-
-let close_process (inchan, outchan) =
-  let pid = find_proc_id "close_process" (Process(inchan, outchan)) in
-  close_in inchan;
-  begin try close_out outchan with Sys_error _ -> () end;
-  snd(waitpid_non_intr pid)
-
-let close_process_full (inchan, outchan, errchan) =
-  let pid =
-    find_proc_id "close_process_full"
-                 (Process_full(inchan, outchan, errchan)) in
+  let proc = Process_full(inchan, outchan, errchan) in
+  let pid = find_proc_id "close_process_full" proc in
+  remove_proc_id proc;
   close_in inchan;
   begin try close_out outchan with Sys_error _ -> () end;
   close_in errchan;
