@@ -13,14 +13,22 @@
 /*                                                                        */
 /**************************************************************************/
 
+#include <caml/config.h>
+
+#ifdef HAS_UNISTD
+#include <unistd.h>
+#endif
+#include <sys/types.h>
+#ifdef HAS_GETAUXVAL
+#include <sys/auxv.h>
+#endif
+
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 
-#ifndef _WIN32
 extern char ** environ;
-#endif
 
-CAMLprim value unix_environment(value unit)
+CAMLprim value unix_environment_unsafe(value unit)
 {
   if (environ != NULL) {
     return caml_copy_string_array((const char**)environ);
@@ -28,3 +36,33 @@ CAMLprim value unix_environment(value unit)
     return Atom(0);
   }
 }
+
+static char **secure_environ(void)
+{
+#ifdef HAS_GETAUXVAL
+  if (!getauxval(AT_SECURE))
+    return environ;
+  else
+   return NULL;
+#elif defined(HAS_ISSETUGID)
+  if (!issetugid ())
+    return environ;
+  else
+    return NULL;
+#else
+  if (geteuid () == getuid () && getegid () == getgid ())
+    return environ;
+  else
+    return NULL;
+#endif
+}
+
+CAMLprim value unix_environment(value unit)
+{
+  char **e = secure_environ();
+  if (e != NULL) {
+    return caml_copy_string_array((const char**)e);
+  } else {
+    return Atom(0);
+  }
+}
