@@ -107,7 +107,7 @@ static LRESULT CALLBACK GraphicsWndProc(HWND hwnd,UINT msg,WPARAM wParam,
                 break;
         }
         caml_gr_handle_event(msg, wParam, lParam);
-        return DefWindowProc(hwnd, msg, wParam, lParam);
+        return DefWindowProcA(hwnd, msg, wParam, lParam);
 }
 
 int DoRegisterClass(void)
@@ -359,8 +359,8 @@ void gr_fail(char *fmt, char *arg)
   if (graphic_failure_exn == NULL) {
     graphic_failure_exn = caml_named_value("Graphics.Graphic_failure");
     if (graphic_failure_exn == NULL)
-      caml_invalid_argument("Exception Graphics.Graphic_failure not initialized, "
-                       "must link graphics.cma");
+      caml_invalid_argument("Exception Graphics.Graphic_failure not "
+                            "initialized, must link graphics.cma");
   }
   sprintf(buffer, fmt, arg);
   caml_raise_with_string(*graphic_failure_exn, buffer);
