@@ -17,26 +17,24 @@ open Asttypes
 open Location
 open Longident
 open Parsetree
+module String = Misc.Stdlib.String
 
 let pp_deps = ref []
 
-module StringSet = Set.Make(struct type t = string let compare = compare end)
-module StringMap = Map.Make(String)
-
 (* Module resolution map *)
 (* Node (set of imports for this path, map for submodules) *)
-type map_tree = Node of StringSet.t * bound_map
-and  bound_map = map_tree StringMap.t
-let bound = Node (StringSet.empty, StringMap.empty)
+type map_tree = Node of String.Set.t * bound_map
+and  bound_map = map_tree String.Map.t
+let bound = Node (String.Set.empty, String.Map.empty)
 
 (*let get_free (Node (s, _m)) = s*)
 let get_map (Node (_s, m)) = m
-let make_leaf s = Node (StringSet.singleton s, StringMap.empty)
-let make_node m =  Node (StringSet.empty, m)
+let make_leaf s = Node (String.Set.singleton s, String.Map.empty)
+let make_node m =  Node (String.Set.empty, m)
 let rec weaken_map s (Node(s0,m0)) =
-  Node (StringSet.union s s0, StringMap.map (weaken_map s) m0)
+  Node (String.Set.union s s0, String.Map.map (weaken_map s) m0)
 let rec collect_free (Node (s, m)) =
-  StringMap.fold (fun _ n -> StringSet.union (collect_free n)) m s
+  String.Map.fold (fun _ n -> String.Set.union (collect_free n)) m s
 
 (* Returns the imports required to access the structure at path p *)
 (* Only raises Not_found if the head of p is not in the toplevel map *)
@@ -44,29 +42,29 @@ let rec lookup_free p m =
   match p with
     [] -> raise Not_found
   | s::p ->
-      let Node (f, m') = StringMap.find s m  in
+      let Node (f, m') = String.Map.find s m  in
       try lookup_free p m' with Not_found -> f
 
 (* Returns the node corresponding to the structure at path p *)
 let rec lookup_map lid m =
   match lid with
-    Lident s    -> StringMap.find s m
-  | Ldot (l, s) -> StringMap.find s (get_map (lookup_map l m))
+    Lident s    -> String.Map.find s m
+  | Ldot (l, s) -> String.Map.find s (get_map (lookup_map l m))
   | Lapply _    -> raise Not_found
 
 (* Collect free module identifiers in the a.s.t. *)
 
-let free_structure_names = ref StringSet.empty
+let free_structure_names = ref String.Set.empty
 
 let add_names s =
-  free_structure_names := StringSet.union s !free_structure_names
+  free_structure_names := String.Set.union s !free_structure_names
 
 let rec add_path bv ?(p=[]) = function
   | Lident s ->
       let free =
-        try lookup_free (s::p) bv with Not_found -> StringSet.singleton s
+        try lookup_free (s::p) bv with Not_found -> String.Set.singleton s
       in
-      (*StringSet.iter (fun s -> Printf.eprintf "%s " s) free;
+      (*String.Set.iter (fun s -> Printf.eprintf "%s " s) free;
         prerr_endline "";*)
       add_names free
   | Ldot(l, s) -> add_path bv ~p:(s::p) l
@@ -76,7 +74,7 @@ let open_module bv lid =
   match lookup_map lid bv with
   | Node (s, m) ->
       add_names s;
-      StringMap.fold StringMap.add m bv
+      String.Map.fold String.Map.add m bv
   | exception Not_found ->
       add_path bv lid; bv
 
@@ -87,7 +85,7 @@ let add_parent bv lid =
 
 let add = add_parent
 
-let addmodule bv lid = add_path bv lid.txt
+let add_module_path bv lid = add_path bv lid.txt
 
 let handle_extension ext =
   match (fst ext).txt with
@@ -106,13 +104,15 @@ let rec add_type bv ty =
   | Ptyp_constr(c, tl) -> add bv c; List.iter (add_type bv) tl
   | Ptyp_object (fl, _) ->
       List.iter
-       (function Otag (_, _, t) -> add_type bv t
+       (fun {pof_desc; _} -> match pof_desc with
+         | Otag (_, t) -> add_type bv t
          | Oinherit t -> add_type bv t) fl
   | Ptyp_class(c, tl) -> add bv c; List.iter (add_type bv) tl
   | Ptyp_alias(t, _) -> add_type bv t
   | Ptyp_variant(fl, _, _) ->
       List.iter
-        (function Rtag(_,_,_,stl) -> List.iter (add_type bv) stl
+        (fun {prf_desc; _} -> match prf_desc with
+          | Rtag(_, _, stl) -> List.iter (add_type bv) stl
           | Rinherit sty -> add_type bv sty)
         fl
   | Ptyp_poly(_, t) -> add_type bv t
@@ -160,34 +160,10 @@ let add_type_extension bv te =
   add bv te.ptyext_path;
   List.iter (add_extension_constructor bv) te.ptyext_constructors
 
-let rec add_class_type bv cty =
-  match cty.pcty_desc with
-    Pcty_constr(l, tyl) ->
-      add bv l; List.iter (add_type bv) tyl
-  | Pcty_signature { pcsig_self = ty; pcsig_fields = fieldl } ->
-      add_type bv ty;
-      List.iter (add_class_type_field bv) fieldl
-  | Pcty_arrow(_, ty1, cty2) ->
-      add_type bv ty1; add_class_type bv cty2
-  | Pcty_extension e -> handle_extension e
-  | Pcty_open (_ovf, m, e) ->
-      let bv = open_module bv m.txt in add_class_type bv e
+let add_type_exception bv te =
+  add_extension_constructor bv te.ptyexn_constructor
 
-and add_class_type_field bv pctf =
-  match pctf.pctf_desc with
-    Pctf_inherit cty -> add_class_type bv cty
-  | Pctf_val(_, _, _, ty) -> add_type bv ty
-  | Pctf_method(_, _, _, ty) -> add_type bv ty
-  | Pctf_constraint(ty1, ty2) -> add_type bv ty1; add_type bv ty2
-  | Pctf_attribute _ -> ()
-  | Pctf_extension e -> handle_extension e
-
-let add_class_description bv infos =
-  add_class_type bv infos.pci_expr
-
-let add_class_type_declaration = add_class_description
-
-let pattern_bv = ref StringMap.empty
+let pattern_bv = ref String.Map.empty
 
 let rec add_pattern bv pat =
   match pat.ppat_desc with
@@ -206,7 +182,7 @@ let rec add_pattern bv pat =
   | Ppat_variant(_, op) -> add_opt add_pattern bv op
   | Ppat_type li -> add bv li
   | Ppat_lazy p -> add_pattern bv p
-  | Ppat_unpack id -> pattern_bv := StringMap.add id.txt bound !pattern_bv
+  | Ppat_unpack id -> pattern_bv := String.Map.add id.txt bound !pattern_bv
   | Ppat_open ( m, p) -> let bv = open_module bv m.txt in add_pattern bv p
   | Ppat_exception p -> add_pattern bv p
   | Ppat_extension e -> handle_extension e
@@ -258,7 +234,7 @@ let rec add_expr bv exp =
   | Pexp_override sel -> List.iter (fun (_s, e) -> add_expr bv e) sel
   | Pexp_letmodule(id, m, e) ->
       let b = add_module_binding bv m in
-      add_expr (StringMap.add id.txt b bv) e
+      add_expr (String.Map.add id.txt b bv) e
   | Pexp_letexception(_, e) -> add_expr bv e
   | Pexp_assert (e) -> add_expr bv e
   | Pexp_lazy (e) -> add_expr bv e
@@ -266,9 +242,14 @@ let rec add_expr bv exp =
   | Pexp_object { pcstr_self = pat; pcstr_fields = fieldl } ->
       let bv = add_pattern bv pat in List.iter (add_class_field bv) fieldl
   | Pexp_newtype (_, e) -> add_expr bv e
-  | Pexp_pack m -> add_module bv m
-  | Pexp_open (_ovf, m, e) ->
-      let bv = open_module bv m.txt in add_expr bv e
+  | Pexp_pack m -> add_module_expr bv m
+  | Pexp_open (o, e) ->
+      let bv = open_declaration bv o in
+      add_expr bv e
+  | Pexp_letop {let_; ands; body} ->
+      let bv' = add_binding_op bv bv let_ in
+      let bv' = List.fold_left (add_binding_op bv) bv' ands in
+      add_expr bv' body
   | Pexp_extension (({ txt = ("ocaml.extension_constructor"|
                               "extension_constructor"); _ },
                      PStr [item]) as e) ->
@@ -293,38 +274,43 @@ and add_bindings recf bv pel =
   List.iter (fun x -> add_expr bv x.pvb_expr) pel;
   bv'
 
+and add_binding_op bv bv' pbop =
+  add_expr bv pbop.pbop_exp;
+  add_pattern bv' pbop.pbop_pat
+
 and add_modtype bv mty =
   match mty.pmty_desc with
     Pmty_ident l -> add bv l
-  | Pmty_alias l -> addmodule bv l
+  | Pmty_alias l -> add_module_path bv l
   | Pmty_signature s -> add_signature bv s
   | Pmty_functor(id, mty1, mty2) ->
       Misc.may (add_modtype bv) mty1;
-      add_modtype (StringMap.add id.txt bound bv) mty2
+      add_modtype (String.Map.add id.txt bound bv) mty2
   | Pmty_with(mty, cstrl) ->
       add_modtype bv mty;
       List.iter
         (function
           | Pwith_type (_, td) -> add_type_declaration bv td
-          | Pwith_module (_, lid) -> addmodule bv lid
+          | Pwith_module (_, lid) -> add_module_path bv lid
           | Pwith_typesubst (_, td) -> add_type_declaration bv td
-          | Pwith_modsubst (_, lid) -> addmodule bv lid
+          | Pwith_modsubst (_, lid) -> add_module_path bv lid
         )
         cstrl
-  | Pmty_typeof m -> add_module bv m
+  | Pmty_typeof m -> add_module_expr bv m
   | Pmty_extension e -> handle_extension e
 
 and add_module_alias bv l =
+  (* If we are in delayed dependencies mode, we delay the dependencies
+       induced by "Lident s" *)
+  (if !Clflags.transparent_modules then add_parent else add_module_path) bv l;
   try
-    add_parent bv l;
     lookup_map l.txt bv
   with Not_found ->
     match l.txt with
       Lident s -> make_leaf s
-    | _ -> addmodule bv l; bound (* cannot delay *)
+    | _ -> add_module_path bv l; bound (* cannot delay *)
 
 and add_modtype_binding bv mty =
-  if not !Clflags.transparent_modules then add_modtype bv mty;
   match mty.pmty_desc with
     Pmty_alias l ->
       add_module_alias bv l
@@ -333,31 +319,36 @@ and add_modtype_binding bv mty =
   | Pmty_typeof modl ->
       add_module_binding bv modl
   | _ ->
-      if !Clflags.transparent_modules then add_modtype bv mty; bound
+      add_modtype bv mty; bound
 
 and add_signature bv sg =
   ignore (add_signature_binding bv sg)
 
 and add_signature_binding bv sg =
-  snd (List.fold_left add_sig_item (bv, StringMap.empty) sg)
+  snd (List.fold_left add_sig_item (bv, String.Map.empty) sg)
 
 and add_sig_item (bv, m) item =
   match item.psig_desc with
     Psig_value vd ->
       add_type bv vd.pval_type; (bv, m)
-  | Psig_type (_, dcls) ->
+  | Psig_type (_, dcls)
+  | Psig_typesubst dcls->
       List.iter (add_type_declaration bv) dcls; (bv, m)
   | Psig_typext te ->
       add_type_extension bv te; (bv, m)
-  | Psig_exception pext ->
-      add_extension_constructor bv pext; (bv, m)
+  | Psig_exception te ->
+      add_type_exception bv te; (bv, m)
   | Psig_module pmd ->
       let m' = add_modtype_binding bv pmd.pmd_type in
-      let add = StringMap.add pmd.pmd_name.txt m' in
+      let add = String.Map.add pmd.pmd_name.txt m' in
+      (add bv, add m)
+  | Psig_modsubst pms ->
+      let m' = add_module_alias bv pms.pms_manifest in
+      let add = String.Map.add pms.pms_name.txt m' in
       (add bv, add m)
   | Psig_recmodule decls ->
       let add =
-        List.fold_right (fun pmd -> StringMap.add pmd.pmd_name.txt bound)
+        List.fold_right (fun pmd -> String.Map.add pmd.pmd_name.txt bound)
                         decls
       in
       let bv' = add bv and m' = add m in
@@ -370,11 +361,11 @@ and add_sig_item (bv, m) item =
       end;
       (bv, m)
   | Psig_open od ->
-      (open_module bv od.popen_lid.txt, m)
+      (open_description bv od, m)
   | Psig_include incl ->
       let Node (s, m') = add_modtype_binding bv incl.pincl_mod in
       add_names s;
-      let add = StringMap.fold StringMap.add m' in
+      let add = String.Map.fold String.Map.add m' in
       (add bv, add m)
   | Psig_class cdl ->
       List.iter (add_class_description bv) cdl; (bv, m)
@@ -385,48 +376,76 @@ and add_sig_item (bv, m) item =
       handle_extension e;
       (bv, m)
 
-and add_module_binding bv modl =
-  if not !Clflags.transparent_modules then add_module bv modl;
-  match modl.pmod_desc with
-    Pmod_ident l ->
-      begin try
-        add_parent bv l;
-        lookup_map l.txt bv
-      with Not_found ->
-        match l.txt with
-          Lident s -> make_leaf s
-        | _ ->  addmodule bv l; bound
-      end
-  | Pmod_structure s ->
-      make_node (snd (add_structure_binding bv s))
-  | _ ->
-      if !Clflags.transparent_modules then add_module bv modl; bound
+and open_description bv od =
+  let Node(s, m) = add_module_alias bv od.popen_expr in
+  add_names s;
+  String.Map.fold String.Map.add m bv
 
-and add_module bv modl =
+and open_declaration bv od =
+  let Node (s, m) = add_module_binding bv od.popen_expr in
+  add_names s;
+  String.Map.fold String.Map.add m bv
+
+and add_module_binding bv modl =
   match modl.pmod_desc with
-    Pmod_ident l -> addmodule bv l
+    Pmod_ident l -> add_module_alias bv l
+  | Pmod_structure s ->
+     make_node (snd @@ add_structure_binding bv s)
+  | _ -> add_module_expr bv modl; bound
+
+and add_module_expr bv modl =
+  match modl.pmod_desc with
+    Pmod_ident l -> add_module_path bv l
   | Pmod_structure s -> ignore (add_structure bv s)
   | Pmod_functor(id, mty, modl) ->
       Misc.may (add_modtype bv) mty;
-      add_module (StringMap.add id.txt bound bv) modl
+      add_module_expr (String.Map.add id.txt bound bv) modl
   | Pmod_apply(mod1, mod2) ->
-      add_module bv mod1; add_module bv mod2
+      add_module_expr bv mod1; add_module_expr bv mod2
   | Pmod_constraint(modl, mty) ->
-      add_module bv modl; add_modtype bv mty
+      add_module_expr bv modl; add_modtype bv mty
   | Pmod_unpack(e) ->
       add_expr bv e
   | Pmod_extension e ->
       handle_extension e
 
+and add_class_type bv cty =
+  match cty.pcty_desc with
+    Pcty_constr(l, tyl) ->
+      add bv l; List.iter (add_type bv) tyl
+  | Pcty_signature { pcsig_self = ty; pcsig_fields = fieldl } ->
+      add_type bv ty;
+      List.iter (add_class_type_field bv) fieldl
+  | Pcty_arrow(_, ty1, cty2) ->
+      add_type bv ty1; add_class_type bv cty2
+  | Pcty_extension e -> handle_extension e
+  | Pcty_open (o, e) ->
+      let bv = open_description bv o in
+      add_class_type bv e
+
+and add_class_type_field bv pctf =
+  match pctf.pctf_desc with
+    Pctf_inherit cty -> add_class_type bv cty
+  | Pctf_val(_, _, _, ty) -> add_type bv ty
+  | Pctf_method(_, _, _, ty) -> add_type bv ty
+  | Pctf_constraint(ty1, ty2) -> add_type bv ty1; add_type bv ty2
+  | Pctf_attribute _ -> ()
+  | Pctf_extension e -> handle_extension e
+
+and add_class_description bv infos =
+  add_class_type bv infos.pci_expr
+
+and add_class_type_declaration bv infos = add_class_description bv infos
+
 and add_structure bv item_list =
   let (bv, m) = add_structure_binding bv item_list in
   add_names (collect_free (make_node m));
   bv
 
 and add_structure_binding bv item_list =
-  List.fold_left add_struct_item (bv, StringMap.empty) item_list
+  List.fold_left add_struct_item (bv, String.Map.empty) item_list
 
-and add_struct_item (bv, m) item : _ StringMap.t * _ StringMap.t =
+and add_struct_item (bv, m) item : _ String.Map.t * _ String.Map.t =
   match item.pstr_desc with
     Pstr_eval (e, _attrs) ->
       add_expr bv e; (bv, m)
@@ -439,19 +458,20 @@ and add_struct_item (bv, m) item : _ StringMap.t * _ StringMap.t =
   | Pstr_typext te ->
       add_type_extension bv te;
       (bv, m)
-  | Pstr_exception pext ->
-      add_extension_constructor bv pext; (bv, m)
+  | Pstr_exception te ->
+      add_type_exception bv te;
+      (bv, m)
   | Pstr_module x ->
       let b = add_module_binding bv x.pmb_expr in
-      let add = StringMap.add x.pmb_name.txt b in
+      let add = String.Map.add x.pmb_name.txt b in
       (add bv, add m)
   | Pstr_recmodule bindings ->
       let add =
-        List.fold_right (fun x -> StringMap.add x.pmb_name.txt bound) bindings
+        List.fold_right (fun x -> String.Map.add x.pmb_name.txt bound) bindings
       in
       let bv' = add bv and m = add m in
       List.iter
-        (fun x -> add_module bv' x.pmb_expr)
+        (fun x -> add_module_expr bv' x.pmb_expr)
         bindings;
       (bv', m)
   | Pstr_modtype x ->
@@ -461,15 +481,20 @@ and add_struct_item (bv, m) item : _ StringMap.t * _ StringMap.t =
       end;
       (bv, m)
   | Pstr_open od ->
-      (open_module bv od.popen_lid.txt, m)
+      (open_declaration bv od, m)
   | Pstr_class cdl ->
       List.iter (add_class_declaration bv) cdl; (bv, m)
   | Pstr_class_type cdtl ->
       List.iter (add_class_type_declaration bv) cdtl; (bv, m)
   | Pstr_include incl ->
-      let Node (s, m') = add_module_binding bv incl.pincl_mod in
-      add_names s;
-      let add = StringMap.fold StringMap.add m' in
+      let Node (s, m') as n = add_module_binding bv incl.pincl_mod in
+      if !Clflags.transparent_modules then
+        add_names s
+      else
+        (* If we are not in the delayed dependency mode, we need to
+           collect all delayed dependencies imported by the include statement *)
+        add_names (collect_free n);
+      let add = String.Map.fold String.Map.add m' in
       (add bv, add m)
   | Pstr_attribute _ -> (bv, m)
   | Pstr_extension (e, _) ->
@@ -480,16 +505,14 @@ and add_use_file bv top_phrs =
   ignore (List.fold_left add_top_phrase bv top_phrs)
 
 and add_implementation bv l =
-  if !Clflags.transparent_modules then
     ignore (add_structure_binding bv l)
-  else ignore (add_structure bv l)
 
 and add_implementation_binding bv l =
   snd (add_structure_binding bv l)
 
 and add_top_phrase bv = function
   | Ptop_def str -> add_structure bv str
-  | Ptop_dir (_, _) -> bv
+  | Ptop_dir _ -> bv
 
 and add_class_expr bv ce =
   match ce.pcl_desc with
@@ -507,8 +530,9 @@ and add_class_expr bv ce =
   | Pcl_constraint(ce, ct) ->
       add_class_expr bv ce; add_class_type bv ct
   | Pcl_extension e -> handle_extension e
-  | Pcl_open (_ovf, m, e) ->
-      let bv = open_module bv m.txt in add_class_expr bv e
+  | Pcl_open (o, e) ->
+      let bv = open_description bv o in
+      add_class_expr bv e
 
 and add_class_field bv pcf =
   match pcf.pcf_desc with
