@@ -17,14 +17,14 @@ open CamlinternalFormatBasics
 open CamlinternalFormat
 
 (* alias to avoid warning for ambiguity between
-   Pervasives.format6
+   Stdlib.format6
    and CamlinternalFormatBasics.format6
 
    (the former is in fact an alias for the latter,
     but the ambiguity warning doesn't care)
 *)
 type ('a, 'b, 'c, 'd, 'e, 'f) format6 =
-  ('a, 'b, 'c, 'd, 'e, 'f) Pervasives.format6
+  ('a, 'b, 'c, 'd, 'e, 'f) Stdlib.format6
 
 
 (* The run-time library for scanners. *)
@@ -39,12 +39,12 @@ module type SCANNING = sig
   type file_name = string
 
   val stdin : in_channel
-  (* The scanning buffer reading from [Pervasives.stdin].
-     [stdib] is equivalent to [Scanning.from_channel Pervasives.stdin]. *)
+  (* The scanning buffer reading from [Stdlib.stdin].
+     [stdib] is equivalent to [Scanning.from_channel Stdlib.stdin]. *)
 
   val stdib : in_channel
   (* An alias for [Scanf.stdin], the scanning buffer reading from
-     [Pervasives.stdin]. *)
+     [Stdlib.stdin]. *)
 
   val next_char : scanbuf -> char
   (* [Scanning.next_char ib] advance the scanning buffer for
@@ -126,11 +126,11 @@ module type SCANNING = sig
   val from_file_bin : file_name -> in_channel
   val from_string : string -> in_channel
   val from_function : (unit -> char) -> in_channel
-  val from_channel : Pervasives.in_channel -> in_channel
+  val from_channel : Stdlib.in_channel -> in_channel
 
   val close_in : in_channel -> unit
 
-  val memo_from_channel : Pervasives.in_channel -> in_channel
+  val memo_from_channel : Stdlib.in_channel -> in_channel
   (* Obsolete. *)
 
 end
@@ -143,8 +143,8 @@ module Scanning : SCANNING = struct
   type file_name = string
 
   type in_channel_name =
-    | From_channel of Pervasives.in_channel
-    | From_file of file_name * Pervasives.in_channel
+    | From_channel of Stdlib.in_channel
+    | From_file of file_name * Stdlib.in_channel
     | From_function
     | From_string
 
@@ -213,7 +213,7 @@ module Scanning : SCANNING = struct
 
   let name_of_input ib =
     match ib.ic_input_name with
-    | From_channel _ic -> "unnamed Pervasives input channel"
+    | From_channel _ic -> "unnamed Stdlib input channel"
     | From_file (fname, _ic) -> fname
     | From_function -> "unnamed function"
     | From_string -> "unnamed character string"
@@ -325,7 +325,7 @@ module Scanning : SCANNING = struct
   let file_buffer_size = ref 1024
 
   (* The scanner closes the input channel at end of input. *)
-  let scan_close_at_end ic = Pervasives.close_in ic; raise End_of_file
+  let scan_close_at_end ic = Stdlib.close_in ic; raise End_of_file
 
   (* The scanner does not close the input channel at end of input:
      it just raises [End_of_file]. *)
@@ -352,13 +352,13 @@ module Scanning : SCANNING = struct
   let from_ic_close_at_end = from_ic scan_close_at_end
   let from_ic_raise_at_end = from_ic scan_raise_at_end
 
-  (* The scanning buffer reading from [Pervasives.stdin].
+  (* The scanning buffer reading from [Stdlib.stdin].
      One could try to define [stdib] as a scanning buffer reading a character
      at a time (no bufferization at all), but unfortunately the top-level
      interaction would be wrong. This is due to some kind of
-     'race condition' when reading from [Pervasives.stdin],
+     'race condition' when reading from [Stdlib.stdin],
      since the interactive compiler and [Scanf.scanf] will simultaneously
-     read the material they need from [Pervasives.stdin]; then, confusion
+     read the material they need from [Stdlib.stdin]; then, confusion
      will result from what should be read by the top-level and what should be
      read by [Scanf.scanf].
      This is even more complicated by the one character lookahead that
@@ -369,7 +369,7 @@ module Scanning : SCANNING = struct
      characters have been read, we simply ask to read more. *)
   let stdin =
     from_ic scan_raise_at_end
-      (From_file ("-", Pervasives.stdin)) Pervasives.stdin
+      (From_file ("-", Stdlib.stdin)) Stdlib.stdin
 
 
   let stdib = stdin
@@ -382,8 +382,8 @@ module Scanning : SCANNING = struct
       from_ic_close_at_end (From_file (fname, ic)) ic
 
 
-  let open_in = open_in_file Pervasives.open_in
-  let open_in_bin = open_in_file Pervasives.open_in_bin
+  let open_in = open_in_file Stdlib.open_in
+  let open_in_bin = open_in_file Stdlib.open_in_bin
 
   let from_file = open_in
   let from_file_bin = open_in_bin
@@ -395,14 +395,14 @@ module Scanning : SCANNING = struct
   let close_in ib =
     match ib.ic_input_name with
     | From_channel ic ->
-      Pervasives.close_in ic
-    | From_file (_fname, ic) -> Pervasives.close_in ic
+      Stdlib.close_in ic
+    | From_file (_fname, ic) -> Stdlib.close_in ic
     | From_function | From_string -> ()
 
 
   (*
      Obsolete: a memo [from_channel] version to build a [Scanning.in_channel]
-     scanning buffer out of a [Pervasives.in_channel].
+     scanning buffer out of a [Stdlib.in_channel].
      This function was used to try to preserve the scanning
      semantics for the (now obsolete) function [fscanf].
      Given that all scanner must read from a [Scanning.in_channel] scanning
@@ -410,8 +410,8 @@ module Scanning : SCANNING = struct
      More precisely, given [ic], all successive calls [fscanf ic] must read
      from the same scanning buffer.
      This obliged this library to allocated scanning buffers that were
-     not properly garbbage collectable, hence leading to memory leaks.
-     If you need to read from a [Pervasives.in_channel] input channel
+     not properly garbage collectable, hence leading to memory leaks.
+     If you need to read from a [Stdlib.in_channel] input channel
      [ic], simply define a [Scanning.in_channel] formatted input channel as in
      [let ib = Scanning.from_channel ic], then use [Scanf.bscanf ib] as usual.
   *)
@@ -556,7 +556,7 @@ let integer_conversion_of_char = function
 
 
 (* Extract an integer literal token.
-   Since the functions Pervasives.*int*_of_string do not accept a leading +,
+   Since the functions Stdlib.*int*_of_string do not accept a leading +,
    we skip it if necessary. *)
 let token_int_literal conv ib =
   let tok =
@@ -655,7 +655,7 @@ let scan_digit_star digitp width ib =
 
 let scan_digit_plus basis digitp width ib =
   (* Ensure we have got enough width left,
-     and read at list one digit. *)
+     and read at least one digit. *)
   if width = 0 then bad_token_length "digits" else
   let c = Scanning.checked_peek_char ib in
   if digitp c then
@@ -1188,7 +1188,7 @@ let stopper_of_formatting_lit fmting =
 
 
 (******************************************************************************)
-                           (* Readers managment *)
+                           (* Reader management *)
 
 (* A call to take_format_readers on a format is evaluated into functions
    taking readers as arguments and aggregate them into an heterogeneous list *)
@@ -1211,7 +1211,7 @@ fun k fmt -> match fmt with
   | Nativeint (_, _, _, rest)        -> take_format_readers k rest
   | Int64 (_, _, _, rest)            -> take_format_readers k rest
   | Float (_, _, _, rest)            -> take_format_readers k rest
-  | Bool rest                        -> take_format_readers k rest
+  | Bool (_, rest)                   -> take_format_readers k rest
   | Alpha rest                       -> take_format_readers k rest
   | Theta rest                       -> take_format_readers k rest
   | Flush rest                       -> take_format_readers k rest
@@ -1284,7 +1284,7 @@ fun k ign fmt -> match ign with
   | Ignored_nativeint (_, _)        -> take_format_readers k fmt
   | Ignored_int64 (_, _)            -> take_format_readers k fmt
   | Ignored_float (_, _)            -> take_format_readers k fmt
-  | Ignored_bool                    -> take_format_readers k fmt
+  | Ignored_bool _                  -> take_format_readers k fmt
   | Ignored_format_arg _            -> take_format_readers k fmt
   | Ignored_format_subst (_, fmtty) -> take_fmtty_format_readers k fmtty fmt
   | Ignored_scan_char_set _         -> take_format_readers k fmt
@@ -1296,7 +1296,7 @@ fun k ign fmt -> match ign with
 
 (* Make a generic scanning function. *)
 (* Scan a stream according to a format and readers obtained by
-   take_format_readers, and aggegate scanned values into an
+   take_format_readers, and aggregate scanned values into an
    heterogeneous list. *)
 (* Return the heterogeneous list of scanned values. *)
 let rec make_scanf : type a c d e f.
@@ -1357,10 +1357,9 @@ fun ib fmt readers -> match fmt with
   | Float ((Float_h | Float_ph | Float_sh | Float_H | Float_pH | Float_sH),
            pad, prec, rest) ->
     pad_prec_scanf ib rest readers pad prec scan_hex_float token_float
-  | Bool rest ->
-    let _ = scan_bool ib in
-    let b = token_bool ib in
-    Cons (b, make_scanf ib rest readers)
+  | Bool (pad, rest) ->
+    let scan _ _ ib = scan_bool ib in
+    pad_prec_scanf ib rest readers pad No_precision scan token_bool
   | Alpha _ ->
     invalid_arg "scanf: bad conversion \"%a\""
   | Theta _ ->
@@ -1372,7 +1371,7 @@ fun ib fmt readers -> match fmt with
     | Cons (reader, readers_rest) ->
         let x = reader ib in
         Cons (x, make_scanf ib fmt_rest readers_rest)
-    | Nil -> 
+    | Nil ->
         invalid_arg "scanf: missing reader"
     end
   | Flush rest ->
@@ -1548,21 +1547,8 @@ let sscanf_format :
   fun s format f -> bscanf_format (Scanning.from_string s) format f
 
 
-let string_to_String s =
-  let l = String.length s in
-  let b = Buffer.create (l + 2) in
-  Buffer.add_char b '\"';
-  for i = 0 to l - 1 do
-    let c = s.[i] in
-    if c = '\"' then Buffer.add_char b '\\';
-    Buffer.add_char b c;
-  done;
-  Buffer.add_char b '\"';
-  Buffer.contents b
-
-
 let format_from_string s fmt =
-  sscanf_format (string_to_String s) fmt (fun x -> x)
+  sscanf_format ("\"" ^ String.escaped s ^ "\"") fmt (fun x -> x)
 
 
 let unescaped s =
