@@ -1,3 +1,12 @@
+(* TEST
+   flags = "-g"
+   ocamlrunparam += ",b=1"
+   * bytecode
+     reference = "${test_source_directory}/raw_backtrace.byte.reference"
+   * native
+     reference = "${test_source_directory}/raw_backtrace.opt.reference"
+     compare_programs = "false"
+*)
 
 (* A test for stack backtraces *)
 
