@@ -1,3 +1,5 @@
+(* TEST *)
+
 (* Test the types nativeint, int32, int64 *)
 
 open Printf
@@ -33,6 +35,7 @@ module type TESTSIG = sig
     val sub: t -> t -> t
     val mul: t -> t -> t
     val div: t -> t -> t
+    val unsigned_div: t -> t -> t
     val rem: t -> t -> t
     val logand: t -> t -> t
     val logor: t -> t -> t
@@ -42,6 +45,7 @@ module type TESTSIG = sig
     val shift_right_logical: t -> int -> t
     val of_int: int -> t
     val to_int: t -> int
+    val unsigned_to_int: t -> int option
     val of_float: float -> t
     val to_float: t -> float
     val zero: t
@@ -53,7 +57,7 @@ module type TESTSIG = sig
     val to_string: t -> string
     val of_string: string -> t
   end
-  val testcomp: t -> t -> bool*bool*bool*bool*bool*bool*int
+  val testcomp: t -> t -> bool*bool*bool*bool*bool*bool*int*int
   val skip_float_tests: bool
 end
 
@@ -70,6 +74,30 @@ struct
     test 4 (to_int (of_int 0x3FFFFFFF)) 0x3FFFFFFF;
     test 5 (to_int (of_int (-0x40000000))) (-0x40000000);
 
+    testing_function "unsigned_to_int";
+    test 1 (unsigned_to_int (of_int 0)) (Some 0);
+    test 2 (unsigned_to_int (of_int 123)) (Some 123);
+    test 3 (unsigned_to_int minus_one)
+      (match Sys.word_size with
+       | 32 -> None
+       | 64 -> Some (int_of_string "0xFFFFFFFF")
+       | _ -> assert false);
+    test 4 (unsigned_to_int max_int)
+      (match Sys.word_size with
+       | 32 -> None
+       | 64 -> Some (to_int max_int)
+       | _ -> assert false);
+    test 5 (unsigned_to_int min_int)
+      (match Sys.word_size with
+       | 32 -> None
+       | 64 -> Some (int_of_string "0x80000000")
+       | _ -> assert false);
+    test 6 (unsigned_to_int (of_int Stdlib.max_int))
+      (match Sys.word_size with
+       | 32 -> Some Stdlib.max_int
+       | 64 -> Some (int_of_string "0xFFFFFFFF")
+       | _ -> assert false);
+
     testing_function "of_string";
     test 1 (of_string "0") (of_int 0);
     test 2 (of_string "123") (of_int 123);
@@ -168,6 +196,21 @@ struct
        11, 1234567, -12345678];
     test 12 (div min_int (of_int (-1))) min_int;
 
+    testing_function "unsigned_div";
+    List.iter
+      (fun (n, a, b, c) -> test n (unsigned_div a b) c)
+      [1, of_int 0, of_int 2, of_int 0;
+       2, of_int 123, of_int 1, of_int 123;
+       3, of_int (-123), of_int 1, of_int (-123);
+       4, of_int (123), of_int (-1), of_int 0;
+       5, of_int (-123), of_int (-1), of_int 0;
+       6, of_int 127531236, of_int 365, of_int (127531236/365);
+       7, of_int 16384, of_int 256, of_int (16384/256);
+       8, of_int (-1), of_int 2, max_int;
+       9, of_int (-1), max_int, of_int 2;
+       10, min_int, of_int 2, shift_left (of_int 1) 30;
+       11, of_int (-1), of_int 8, shift_right_logical (of_int (-1)) 3];
+
     testing_function "mod";
     List.iter
       (fun (n, a, b) -> test n (rem (of_int a) (of_int b)) (of_int (a mod b)))
@@ -269,19 +312,19 @@ struct
 
     testing_function "Comparisons";
     test 1 (testcomp (of_int 0) (of_int 0))
-           (true,false,false,false,true,true,0);
+           (true,false,false,false,true,true,0,0);
     test 2 (testcomp (of_int 1234567) (of_int 1234567))
-           (true,false,false,false,true,true,0);
+           (true,false,false,false,true,true,0, 0);
     test 3 (testcomp (of_int 0) (of_int 1))
-           (false,true,true,false,true,false,-1);
+           (false,true,true,false,true,false,-1,-1);
     test 4 (testcomp (of_int (-1)) (of_int 0))
-           (false,true,true,false,true,false,-1);
+           (false,true,true,false,true,false,-1,1);
     test 5 (testcomp (of_int 1) (of_int 0))
-           (false,true,false,true,false,true,1);
+           (false,true,false,true,false,true,1,1);
     test 6 (testcomp (of_int 0) (of_int (-1)))
-           (false,true,false,true,false,true,1);
+           (false,true,false,true,false,true,1,-1);
     test 7 (testcomp max_int min_int)
-           (false,true,false,true,false,true,1);
+           (false,true,false,true,false,true,1,-1);
 
     ()
 end
@@ -301,6 +344,15 @@ struct
     test 4 (to_int (of_int 0x3FFFFFFF)) 0x3FFFFFFF;
     test 5 (to_int (of_int (-0x40000000))) (-0x40000000);
 
+    testing_function "unsigned_to_int";
+    test 1 (unsigned_to_int (of_int 0)) (Some 0);
+    test 2 (unsigned_to_int (of_int 123)) (Some 123);
+    test 3 (unsigned_to_int minus_one) None;
+    test 4 (unsigned_to_int max_int) None;
+    test 5 (unsigned_to_int min_int) None;
+    test 6 (unsigned_to_int (of_int Stdlib.max_int))
+      (Some Stdlib.max_int);
+
     testing_function "of_string";
     test 1 (of_string "0") (of_int 0);
     test 2 (of_string "123") (of_int 123);
@@ -404,6 +456,21 @@ struct
        11, 1234567, -12345678];
     test 12 (div min_int (of_int (-1))) min_int;
 
+    testing_function "unsigned_div";
+    List.iter
+      (fun (n, a, b, c) -> test n (unsigned_div a b) c)
+      [1, of_int 0, of_int 2, of_int 0;
+       2, of_int 123, of_int 1, of_int 123;
+       3, of_int (-123), of_int 1, of_int (-123);
+       4, of_int (123), of_int (-1), of_int 0;
+       5, of_int (-123), of_int (-1), of_int 0;
+       6, of_int 127531236, of_int 365, of_int (127531236/365);
+       7, of_int 16384, of_int 256, of_int (16384/256);
+       8, of_int (-1), of_int 2, max_int;
+       9, of_int (-1), max_int, of_int 2;
+       10, min_int, of_int 2, shift_left (of_int 1) 62;
+       11, of_int (-1), of_int 8, shift_right_logical (of_int (-1)) 3];
+
     testing_function "mod";
     List.iter
       (fun (n, a, b) -> test n (rem (of_int a) (of_int b)) (of_int (a mod b)))
@@ -487,19 +554,19 @@ struct
 
     testing_function "Comparisons";
     test 1 (testcomp (of_int 0) (of_int 0))
-           (true,false,false,false,true,true,0);
+           (true,false,false,false,true,true,0,0);
     test 2 (testcomp (of_int 1234567) (of_int 1234567))
-           (true,false,false,false,true,true,0);
+           (true,false,false,false,true,true,0,0);
     test 3 (testcomp (of_int 0) (of_int 1))
-           (false,true,true,false,true,false,-1);
+           (false,true,true,false,true,false,-1,-1);
     test 4 (testcomp (of_int (-1)) (of_int 0))
-           (false,true,true,false,true,false,-1);
+           (false,true,true,false,true,false,-1,1);
     test 5 (testcomp (of_int 1) (of_int 0))
-           (false,true,false,true,false,true,1);
+           (false,true,false,true,false,true,1,1);
     test 6 (testcomp (of_int 0) (of_int (-1)))
-           (false,true,false,true,false,true,1);
+           (false,true,false,true,false,true,1,-1);
     test 7 (testcomp max_int min_int)
-           (false,true,false,true,false,true,1);
+           (false,true,false,true,false,true,1,-1);
 
     ()
 end
@@ -507,11 +574,14 @@ end
 (******** The test proper **********)
 
 let testcomp_int32 (a : int32) (b : int32) =
-  (a = b, a <> b, a < b, a > b, a <= b, a >= b, compare a b)
+  (a = b, a <> b, a < b, a > b, a <= b, a >= b, compare a b,
+   Int32.unsigned_compare a b)
 let testcomp_int64 (a : int64) (b : int64) =
-  (a = b, a <> b, a < b, a > b, a <= b, a >= b, compare a b)
+  (a = b, a <> b, a < b, a > b, a <= b, a >= b, compare a b,
+   Int64.unsigned_compare a b)
 let testcomp_nativeint (a : nativeint) (b : nativeint) =
-  (a = b, a <> b, a < b, a > b, a <= b, a >= b, compare a b)
+  (a = b, a <> b, a < b, a > b, a <= b, a >= b, compare a b,
+   Nativeint.unsigned_compare a b)
 
 let _ =
   testing_function "-------- Int32 --------";
