@@ -20,21 +20,21 @@
 value mycallback1(value fun, value arg)
 {
   value res;
-  res = callback(fun, arg);
+  res = caml_callback(fun, arg);
   return res;
 }
 
 value mycallback2(value fun, value arg1, value arg2)
 {
   value res;
-  res = callback2(fun, arg1, arg2);
+  res = caml_callback2(fun, arg1, arg2);
   return res;
 }
 
 value mycallback3(value fun, value arg1, value arg2, value arg3)
 {
   value res;
-  res = callback3(fun, arg1, arg2, arg3);
+  res = caml_callback3(fun, arg1, arg2, arg3);
   return res;
 }
 
@@ -46,14 +46,14 @@ value mycallback4(value fun, value arg1, value arg2, value arg3, value arg4)
   args[1] = arg2;
   args[2] = arg3;
   args[3] = arg4;
-  res = callbackN(fun, 4, args);
+  res = caml_callbackN(fun, 4, args);
   return res;
 }
 
 value mypushroot(value v, value fun, value arg)
 {
   Begin_root(v)
-    callback(fun, arg);
+    caml_callback(fun, arg);
   End_roots();
   return v;
 }
@@ -63,7 +63,7 @@ value mycamlparam (value v, value fun, value arg)
   CAMLparam3 (v, fun, arg);
   CAMLlocal2 (x, y);
   x = v;
-  y = callback (fun, arg);
+  y = caml_callback (fun, arg);
   v = x;
   CAMLreturn (v);
 }
