@@ -1,3 +1,8 @@
+(* TEST
+   flags += " -w a "
+   modules = "globrootsprim.c"
+*)
+
 module type GLOBREF = sig
   type t
   val register: string -> t
@@ -26,9 +31,9 @@ module Test(G: GLOBREF) = struct
 
   let size = 1024
 
-  let vals = Array.init size string_of_int
+  let vals = Array.init size Int.to_string
 
-  let a = Array.init size (fun i -> G.register (string_of_int i))
+  let a = Array.init size (fun i -> G.register (Int.to_string i))
 
   let check () =
     for i = 0 to size - 1 do
@@ -46,14 +51,14 @@ module Test(G: GLOBREF) = struct
         Gc.minor()
     | 5|6|7|8|9|10|11|12 ->             (* update with young value *)
         let i = Random.int size in
-        G.set a.(i) (string_of_int i)
+        G.set a.(i) (Int.to_string i)
     | 13|14|15|16|17|18|19|20 ->        (* update with old value *)
         let i = Random.int size in
         G.set a.(i) vals.(i)
     | 21|22|23|24|25|26|27|28 ->        (* re-register young value *)
         let i = Random.int size in
         G.remove a.(i);
-        a.(i) <- G.register (string_of_int i)
+        a.(i) <- G.register (Int.to_string i)
     | (*29|30|31|32|33|34|35|36*) _ ->  (* re-register old value *)
         let i = Random.int size in
         G.remove a.(i);
