@@ -1,3 +1,8 @@
+(* TEST
+   * hasunix
+   include unix
+*)
+
 open Bigarray
 
 (* Test harness *)
