@@ -1,3 +1,6 @@
+(* TEST
+*)
+
 (* Hashtable operations, using maps as a reference *)
 
 open Printf
@@ -30,6 +33,22 @@ module Test(H: Hashtbl.S) (M: Map.S with type key = H.key) = struct
       true
     with Exit | Not_found -> false
 
+  let to_list_ h : _ list =
+    H.fold (fun k v acc -> (k,v) :: acc) h []
+    |> List.sort Stdlib.compare
+
+  let check_to_seq h =
+    let l = to_list_ h in
+    let l2 = List.of_seq (H.to_seq h) in
+    assert (l = List.sort Stdlib.compare l2)
+
+  let check_to_seq_of_seq h =
+    let h' = H.create (H.length h) in
+    H.add_seq h' (H.to_seq h);
+    (*printf "h.len=%d, h'.len=%d\n" (List.length @@ to_list_ h)
+                                     (List.length @@ to_list_ h');*)
+    assert (to_list_ h = to_list_ h')
+
   let test data =
     let n = Array.length data in
     let h = H.create 51 and m = ref M.empty in
@@ -39,6 +58,8 @@ module Test(H: Hashtbl.S) (M: Map.S with type key = H.key) = struct
       data;
     printf "Insertion: %s\n"
            (if incl_mh !m h && domain_hm h !m then "passed" else "FAILED");
+    check_to_seq_of_seq h;
+    check_to_seq h;
     (* Insert all data with H.replace *)
     H.clear h; m := M.empty;
     Array.iter
@@ -46,42 +67,47 @@ module Test(H: Hashtbl.S) (M: Map.S with type key = H.key) = struct
       data;
     printf "Insertion: %s\n"
            (if incl_mh !m h && incl_hm h !m then "passed" else "FAILED");
+    check_to_seq_of_seq h;
+    check_to_seq h;
     (* Remove some of the data *)
     for i = 0 to n/3 - 1 do
       let (k, _) = data.(i) in H.remove h k; m := M.remove k !m
     done;
     printf "Removal: %s\n"
-           (if incl_mh !m h && incl_hm h !m then "passed" else "FAILED")
+      (if incl_mh !m h && incl_hm h !m then "passed" else "FAILED");
+    check_to_seq_of_seq h;
+    check_to_seq h;
+    ()
 
 end
 
 module SS = struct
   type t = string
-  let compare (x:t) (y:t) = Pervasives.compare x y
+  let compare (x:t) (y:t) = Stdlib.compare x y
   let equal (x:t) (y:t) = x=y
   let hash = Hashtbl.hash
 end
 module SI = struct
   type t = int
-  let compare (x:t) (y:t) = Pervasives.compare x y
+  let compare (x:t) (y:t) = Stdlib.compare x y
   let equal (x:t) (y:t) = x=y
   let hash = Hashtbl.hash
 end
 module SSP = struct
   type t = string*string
-  let compare (x:t) (y:t) = Pervasives.compare x y
+  let compare (x:t) (y:t) = Stdlib.compare x y
   let equal (x:t) (y:t) = x=y
   let hash = Hashtbl.hash
 end
 module SSL = struct
   type t = string list
-  let compare (x:t) (y:t) = Pervasives.compare x y
+  let compare (x:t) (y:t) = Stdlib.compare x y
   let equal (x:t) (y:t) = x=y
   let hash = Hashtbl.hash
 end
 module SSA = struct
   type t = string array
-  let compare (x:t) (y:t) = Pervasives.compare x y
+  let compare (x:t) (y:t) = Stdlib.compare x y
   let equal (x:t) (y:t) = x=y
   let hash = Hashtbl.hash
 end
@@ -115,6 +141,12 @@ module HofM (M: Map.S) : Hashtbl.S with type key = M.key =
     let length = Hashtbl.length
     let stats = Hashtbl.stats
     let filter_map_inplace = Hashtbl.filter_map_inplace
+    let to_seq = Hashtbl.to_seq
+    let to_seq_keys = Hashtbl.to_seq_keys
+    let to_seq_values = Hashtbl.to_seq_values
+    let of_seq = Hashtbl.of_seq
+    let add_seq = Hashtbl.add_seq
+    let replace_seq = Hashtbl.replace_seq
   end
 
 module HS1 = HofM(MS)
@@ -202,7 +234,7 @@ let list_data data =
       hd :: tl
     end in
   for i = 0 to Array.length d - 1 do
-    d.(i) <- (mklist (Random.int 16), string_of_int i)
+    d.(i) <- (mklist (Random.int 16), Int.to_string i)
   done;
   d
 
