@@ -1,3 +1,12 @@
+(* TEST
+
+* hassysthreads
+include systhreads
+** bytecode
+** native
+
+*)
+
 (* Torture test - I/O interspersed with lots of GC *)
 
 let finished = ref false
@@ -26,7 +35,8 @@ let reader_thread (ic, size) =
 (*    print_string "reader "; print_int n; print_newline(); *)
     for i = 0 to n-1 do
       if Bytes.get buff i = 'b' then Thread.exit()
-      else if Bytes.get buff i <> 'a' then print_string "error in reader_thread\n"
+      else if Bytes.get buff i <> 'a' then
+        print_string "error in reader_thread\n"
     done
   done
 
