@@ -1,3 +1,11 @@
+(* TEST
+include tool-ocaml-lib
+flags = "-w a"
+ocaml_script_as_argument = "true"
+* setup-ocaml-build-env
+** ocaml
+*)
+
 open Lib;;
 type t = {
   a : int;
@@ -11,19 +19,19 @@ if { a = 7; b = 6; c = 5; d = 4; e = 3 }.e <> 3 then raise Not_found;;
 
 (**
        0 CONSTINT 42
-       2 PUSHACC0 
+       2 PUSHACC0
        3 MAKEBLOCK1 0
        5 POP 1
        7 SETGLOBAL Lib
-       9 CONST3 
+       9 CONST3
       10 PUSHGETGLOBAL <0>(7, 6, 5, 4, 3)
       12 GETFIELD 4
-      14 NEQ 
+      14 NEQ
       15 BRANCHIFNOT 22
       17 GETGLOBAL Not_found
       19 MAKEBLOCK1 0
-      21 RAISE 
-      22 ATOM0 
+      21 RAISE
+      22 ATOM0
       23 SETGLOBAL T201-getfield
-      25 STOP 
+      25 STOP
 **)
