@@ -1,3 +1,7 @@
+(* TEST
+   * expect
+*)
+
 type 'a t = [< `Foo | `Bar] as 'a;;
 type 'a s = [< `Foo | `Bar | `Baz > `Bar] as 'a;;
 
@@ -17,7 +21,9 @@ type 'a first = First : 'b t second -> ([< `Bar | `Foo ] as 'b) t first
 and 'a second = Second : [< `Bar | `Baz | `Foo > `Bar ] s second
 type aux = Aux : ([< `Bar | `Foo ] as 'a) t second * ('a -> int) -> aux
 val it : [< `Bar | `Foo > `Bar ] = `Bar
-Line _, characters 27-29:
+Line 11, characters 27-29:
+11 | let g (Aux(Second, f)) = f it;;
+                                ^^
 Error: This expression has type [< `Bar | `Foo > `Bar ]
        but an expression was expected of type [< `Bar | `Foo ]
        Types for tag `Bar are incompatible
