@@ -1,3 +1,7 @@
+(* TEST
+   * expect
+*)
+
 type bar = < bar: unit >
 
 type _ ty = Int : int ty
@@ -15,7 +19,13 @@ class foo =
 type bar = < bar : unit >
 type _ ty = Int : int ty
 type dyn = Dyn : 'a ty -> dyn
-Line _, characters 0-108:
+Line 7, characters 0-108:
+ 7 | class foo =
+ 8 |   object (this)
+ 9 |     method foo (Dyn ty) =
+10 |       match ty with
+11 |       | Int -> (this :> bar)
+12 |   end.................................
 Error: This class should be virtual.
        The following methods are undefined : bar
 |}];;
