@@ -1,3 +1,7 @@
+(* TEST
+   * expect
+*)
+
 module Exp =
   struct
 
@@ -99,11 +103,16 @@ module Nonexhaustive =
   end
 ;;
 [%%expect{|
-Line _, characters 6-34:
+Line 11, characters 6-34:
+11 | ......function
+12 |         | C2 x -> x
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 C1 _
-Line _, characters 6-77:
+Line 24, characters 6-77:
+24 | ......function
+25 |         | Foo _ , Foo _ -> true
+26 |         | Bar _, Bar _ -> true
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 (Bar _, Foo _)
@@ -148,11 +157,15 @@ module PR6862 = struct
   class d (Just x) = object method x : int = x end
 end;;
 [%%expect{|
-Line _, characters 10-18:
+Line 2, characters 10-18:
+2 |   class c (Some x) = object method x : int = x end
+              ^^^^^^^^
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 None
-Line _, characters 10-18:
+Line 4, characters 10-18:
+4 |   class d (Just x) = object method x : int = x end
+              ^^^^^^^^
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Nothing
@@ -179,7 +192,9 @@ module PR6220 = struct
   let g : int t -> int = function I -> 1 | _ -> 2 (* warn *)
 end;;
 [%%expect{|
-Line _, characters 43-44:
+Line 4, characters 43-44:
+4 |   let g : int t -> int = function I -> 1 | _ -> 2 (* warn *)
+                                               ^
 Warning 56: this match case is unreachable.
 Consider replacing it with a refutation case '<pat> -> .'
 module PR6220 :
@@ -245,7 +260,9 @@ module PR6801 = struct
     | String s -> print_endline s (* warn : Any *)
 end;;
 [%%expect{|
-Line _, characters 4-50:
+Line 8, characters 4-50:
+8 | ....match x with
+9 |     | String s -> print_endline s.................
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Any
@@ -267,7 +284,9 @@ module Existential_escape =
   end
 ;;
 [%%expect{|
-Line _, characters 21-22:
+Line 5, characters 21-22:
+5 |     let eval (D x) = x
+                         ^
 Error: This expression has type $D_'a t
        but an expression was expected of type 'a
        The type constructor $D_'a would escape its scope
@@ -298,10 +317,11 @@ module Or_patterns =
 end
 ;;
 [%%expect{|
-Line _, characters 11-19:
-Error: This pattern matches values of type int t
-       but a pattern was expected which matches values of type s t
-       Type int is not compatible with type s
+module Or_patterns :
+  sig
+    type _ t = IntLit : int -> int t | BoolLit : bool -> bool t
+    val eval : 's t -> unit
+  end
 |}];;
 
 module Polymorphic_variants =
@@ -347,7 +367,9 @@ module Propagation :
     val check : 's t -> 's
   end
 |}, Principal{|
-Line _, characters 19-20:
+Line 13, characters 19-20:
+13 |     | BoolLit b -> b
+                        ^
 Error: This expression has type bool but an expression was expected of type s
 |}];;
 
@@ -358,11 +380,15 @@ module Normal_constrs = struct
   let f = function A -> 1 | B -> 2
 end;;
 [%%expect{|
-Line _, characters 28-29:
+Line 5, characters 28-29:
+5 |   let f = function A -> 1 | B -> 2
+                                ^
 Error: This variant pattern is expected to have type a
        The constructor B does not belong to type a
 |}, Principal{|
-Line _, characters 28-29:
+Line 5, characters 28-29:
+5 |   let f = function A -> 1 | B -> 2
+                                ^
 Error: This pattern matches values of type b
        but a pattern was expected which matches values of type a
 |}];;
@@ -374,7 +400,9 @@ module PR6849 = struct
       Foo -> 5
 end;;
 [%%expect{|
-Line _, characters 6-9:
+Line 5, characters 6-9:
+5 |       Foo -> 5
+          ^^^
 Error: This pattern matches values of type 'a t
        but a pattern was expected which matches values of type int
 |}];;
@@ -403,7 +431,9 @@ let test : type a. a t -> _ =
   function Int -> ky (1 : a) 1  (* fails *)
 ;;
 [%%expect{|
-Line _, characters 18-30:
+Line 2, characters 18-30:
+2 |   function Int -> ky (1 : a) 1  (* fails *)
+                      ^^^^^^^^^^^^
 Error: This expression has type a = int
        but an expression was expected of type 'a
        This instance of int is ambiguous:
@@ -415,7 +445,9 @@ let test : type a. a t -> a = fun x ->
   in r
 ;;
 [%%expect{|
-Line _, characters 30-42:
+Line 2, characters 30-42:
+2 |   let r = match x with Int -> ky (1 : a) 1  (* fails *)
+                                  ^^^^^^^^^^^^
 Error: This expression has type a = int
        but an expression was expected of type 'a
        This instance of int is ambiguous:
@@ -427,7 +459,9 @@ let test : type a. a t -> a = fun x ->
   in r
 ;;
 [%%expect{|
-Line _, characters 30-42:
+Line 2, characters 30-42:
+2 |   let r = match x with Int -> ky 1 (1 : a)  (* fails *)
+                                  ^^^^^^^^^^^^
 Error: This expression has type a = int
        but an expression was expected of type 'a
        This instance of int is ambiguous:
@@ -501,7 +535,9 @@ let test2 : type a. a t -> a option = fun x ->
   !u
 ;; (* fails because u : (int | a) option ref *)
 [%%expect{|
-Line _, characters 46-48:
+Line 4, characters 46-48:
+4 |   begin match x with Int -> u := Some 1; r := !u end;
+                                                  ^^
 Error: This expression has type int option
        but an expression was expected of type a option
        Type int is not compatible with type a = int
@@ -537,7 +573,9 @@ let we_y1x (type a) (x : a) (v : a t) =
 ;; (* fail *)
 [%%expect{|
 val either : 'a -> 'a -> 'a = <fun>
-Line _, characters 44-45:
+Line 3, characters 44-45:
+3 |   match v with Int -> let y = either 1 x in y
+                                                ^
 Error: This expression has type a = int
        but an expression was expected of type 'a
        This instance of int is ambiguous:
@@ -596,7 +634,9 @@ let f (type a) (x : a t) y =
     in M.z
 ;; (* fails because of aliasing... *)
 [%%expect{|
-Line _, characters 46-47:
+Line 3, characters 46-47:
+3 |     let module M = struct type b = a let z = (y : b) end
+                                                  ^
 Error: This expression has type a = int
        but an expression was expected of type b = int
        This instance of int is ambiguous:
@@ -647,7 +687,9 @@ let f : type a b. (a,b) eq -> (<m : a; ..> as 'c) -> (<m : b; ..> as 'c) =
 ;; (* fail *)
 [%%expect{|
 type (_, _) eq = Eq : ('a, 'a) eq
-Line _, characters 4-90:
+Line 3, characters 4-90:
+3 | ....f : type a b. (a,b) eq -> (<m : a; ..> as 'c) -> (<m : b; ..> as 'c) =
+4 |   fun Eq o -> o
 Error: The universal type variable 'b cannot be generalized:
        it is already bound to another variable.
 |}];;
@@ -656,7 +698,9 @@ let f : type a b. (a,b) eq -> <m : a; ..> -> <m : b; ..> =
   fun Eq o -> o
 ;; (* fail *)
 [%%expect{|
-Line _, characters 14-15:
+Line 2, characters 14-15:
+2 |   fun Eq o -> o
+                  ^
 Error: This expression has type < m : a; .. >
        but an expression was expected of type < m : b; .. >
        Type a is not compatible with type b = a
@@ -667,7 +711,9 @@ Error: This expression has type < m : a; .. >
 let f (type a) (type b) (eq : (a,b) eq) (o : <m : a; ..>) : <m : b; ..> =
   match eq with Eq -> o ;; (* should fail *)
 [%%expect{|
-Line _, characters 22-23:
+Line 2, characters 22-23:
+2 |   match eq with Eq -> o ;; (* should fail *)
+                          ^
 Error: This expression has type < m : a; .. >
        but an expression was expected of type < m : b; .. >
        Type a is not compatible with type b = a
@@ -705,7 +751,9 @@ let f : type a b. (a,b) eq -> < m : a; .. > -> < m : b > =
 [%%expect{|
 val f : ('a, 'b) eq -> < m : 'a > -> < m : 'b > = <fun>
 |}, Principal{|
-Line _, characters 44-45:
+Line 4, characters 44-45:
+4 |     let r : < m : b > = match eq with Eq -> o in (* fail with principal *)
+                                                ^
 Error: This expression has type < m : a >
        but an expression was expected of type < m : b >
        Type a is not compatible with type b = a
@@ -719,7 +767,9 @@ let f : type a b. (a,b) eq -> < m : a; .. > -> < m : b > =
     ignore (o : < m : a >);
     r;;
 [%%expect{|
-Line _, characters 44-45:
+Line 3, characters 44-45:
+3 |     let r : < m : b > = match eq with Eq -> o in (* fail *)
+                                                ^
 Error: This expression has type < m : a; .. >
        but an expression was expected of type < m : b >
        Type a is not compatible with type b = a
@@ -730,18 +780,29 @@ Error: This expression has type < m : a; .. >
 let f : type a b. (a,b) eq -> [> `A of a] -> [> `A of b] =
   fun Eq o -> o ;; (* fail *)
 [%%expect{|
-Line _, characters 14-15:
+Line 2, characters 14-15:
+2 |   fun Eq o -> o ;; (* fail *)
+                  ^
 Error: This expression has type [> `A of a ]
        but an expression was expected of type [> `A of b ]
        Type a is not compatible with type b = a
        This instance of a is ambiguous:
        it would escape the scope of its equation
+|}, Principal{|
+Line 2, characters 9-15:
+2 |   fun Eq o -> o ;; (* fail *)
+             ^^^^^^
+Error: This expression has type ([> `A of b ] as 'a) -> 'a
+       but an expression was expected of type [> `A of a ] -> [> `A of b ]
+       Types for tag `A are incompatible
 |}];;
 
 let f (type a b) (eq : (a,b) eq) (v : [> `A of a]) : [> `A of b] =
   match eq with Eq -> v ;; (* should fail *)
 [%%expect{|
-Line _, characters 22-23:
+Line 2, characters 22-23:
+2 |   match eq with Eq -> v ;; (* should fail *)
+                          ^
 Error: This expression has type [> `A of a ]
        but an expression was expected of type [> `A of b ]
        Type a is not compatible with type b = a
@@ -752,7 +813,9 @@ Error: This expression has type [> `A of a ]
 let f : type a b. (a,b) eq -> [< `A of a | `B] -> [< `A of b | `B] =
   fun Eq o -> o ;; (* fail *)
 [%%expect{|
-Line _, characters 4-84:
+Line 1, characters 4-84:
+1 | ....f : type a b. (a,b) eq -> [< `A of a | `B] -> [< `A of b | `B] =
+2 |   fun Eq o -> o..............
 Error: This definition has type
          ('a, 'b) eq -> ([< `A of 'b & 'a | `B ] as 'c) -> 'c
        which is less general than 'a0 'b0. ('a0, 'b0) eq -> 'c -> 'c
@@ -779,7 +842,9 @@ let f : type a b. (a,b) eq -> [> `A of a | `B] -> [`A of b | `B] =
 [%%expect{|
 val f : ('a, 'b) eq -> [ `A of 'a | `B ] -> [ `A of 'b | `B ] = <fun>
 |}, Principal{|
-Line _, characters 49-50:
+Line 4, characters 49-50:
+4 |     let r : [`A of b | `B] = match eq with Eq -> o in (* fail with principal *)
+                                                     ^
 Error: This expression has type [ `A of a | `B ]
        but an expression was expected of type [ `A of b | `B ]
        Type a is not compatible with type b = a
@@ -793,7 +858,9 @@ let f : type a b. (a,b) eq -> [> `A of a | `B] -> [`A of b | `B] =
     ignore (o : [< `A of a | `B]);
     r;;
 [%%expect{|
-Line _, characters 49-50:
+Line 3, characters 49-50:
+3 |     let r : [`A of b | `B] = match eq with Eq -> o in (* fail *)
+                                                     ^
 Error: This expression has type [> `A of a | `B ]
        but an expression was expected of type [ `A of b | `B ]
        Type a is not compatible with type b = a
@@ -848,7 +915,14 @@ let f : type a. a ty -> a t -> int = fun x y ->
   | TA, D z -> z
 ;; (* warn *)
 [%%expect{|
-Line _, characters 2-153:
+Line 2, characters 2-153:
+2 | ..match x, y with
+3 |   | _, A z -> z
+4 |   | _, B z -> if z then 1 else 2
+5 |   | _, C z -> truncate z
+6 |   | TE TC, D [|1.0|] -> 14
+7 |   | TA, D 0 -> -1
+8 |   | TA, D z -> z
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 (TE TC, D [| 0. |])
@@ -865,7 +939,9 @@ let f : type a. a ty -> a t -> int = fun x y ->
   | D z, TA -> z
 ;; (* fail *)
 [%%expect{|
-Line _, characters 6-13:
+Line 6, characters 6-13:
+6 |   | D [|1.0|], TE TC -> 14
+          ^^^^^^^
 Error: This pattern matches values of type 'a array
        but a pattern was expected which matches values of type a
 |}];;
@@ -883,7 +959,9 @@ let f : type a. a ty -> a t -> int = fun x y ->
 ;; (* fail *)
 [%%expect{|
 type ('a, 'b) pair = { right : 'a; left : 'b; }
-Line _, characters 25-32:
+Line 8, characters 25-32:
+8 |   | {left=TE TC; right=D [|1.0|]} -> 14
+                             ^^^^^^^
 Error: This pattern matches values of type 'a array
        but a pattern was expected which matches values of type a
 |}];;
@@ -901,7 +979,14 @@ let f : type a. a ty -> a t -> int = fun x y ->
 ;; (* ok *)
 [%%expect{|
 type ('a, 'b) pair = { left : 'a; right : 'b; }
-Line _, characters 2-244:
+Line 4, characters 2-244:
+ 4 | ..match {left=x; right=y} with
+ 5 |   | {left=_; right=A z} -> z
+ 6 |   | {left=_; right=B z} -> if z then 1 else 2
+ 7 |   | {left=_; right=C z} -> truncate z
+ 8 |   | {left=TE TC; right=D [|1.0|]} -> 14
+ 9 |   | {left=TA; right=D 0} -> -1
+10 |   | {left=TA; right=D z} -> z
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 {left=TE TC; right=D [| 0. |]}
@@ -919,7 +1004,9 @@ let f : type a b. (a M.t, b M.t) eq -> (a, b) eq =
 ;;
 [%%expect{|
 module M : sig type 'a t val eq : ('a t, 'b t) eq end
-Line _, characters 17-19:
+Line 6, characters 17-19:
+6 |   function Eq -> Eq (* fail *)
+                     ^^
 Error: This expression has type (a, a) eq
        but an expression was expected of type (a, b) eq
        Type a is not compatible with type b
@@ -973,7 +1060,9 @@ let g (type t) (x:t) (e : t int_foo) (e' : t int_bar) =
 [%%expect{|
 type _ int_foo = IF_constr : < foo : int; .. > int_foo
 type _ int_bar = IB_constr : < bar : int; .. > int_bar
-Line _, characters 3-4:
+Line 10, characters 3-4:
+10 |   (x:<foo:int>)
+        ^
 Error: This expression has type t = < foo : int; .. >
        but an expression was expected of type < foo : int >
        Type $0 = < bar : int; .. > is not compatible with type <  >
@@ -985,7 +1074,9 @@ let g (type t) (x:t) (e : t int_foo) (e' : t int_bar) =
   (x:<foo:int;bar:int>)
 ;;
 [%%expect{|
-Line _, characters 3-4:
+Line 3, characters 3-4:
+3 |   (x:<foo:int;bar:int>)
+       ^
 Error: This expression has type t = < foo : int; .. >
        but an expression was expected of type < bar : int; foo : int >
        Type $0 = < bar : int; .. > is not compatible with type < bar : int >
@@ -997,7 +1088,9 @@ let g (type t) (x:t) (e : t int_foo) (e' : t int_bar) =
   (x:<foo:int;bar:int;..>)
 ;;
 [%%expect{|
-Line _, characters 2-26:
+Line 3, characters 2-26:
+3 |   (x:<foo:int;bar:int;..>)
+      ^^^^^^^^^^^^^^^^^^^^^^^^
 Error: This expression has type < bar : int; foo : int; .. >
        but an expression was expected of type 'a
        The type constructor $1 would escape its scope
@@ -1070,3 +1163,69 @@ let f : type a b. (a,b) eq -> (b,int) eq -> a -> b -> _ = fun ab bint a b ->
 [%%expect{|
 val f : ('a, 'b) eq -> ('b, int) eq -> 'a -> 'b -> unit = <fun>
 |}];;
+
+let f : type a b. (a,b) eq -> (a,int) eq -> a -> b -> _ = fun ab aint a b ->
+  let Eq = aint in
+  let x =
+    let Eq = ab in
+    if true then a else b
+  in ignore x
+;; (* ok *)
+[%%expect{|
+Line 5, characters 24-25:
+5 |     if true then a else b
+                            ^
+Error: This expression has type b = int
+       but an expression was expected of type a = int
+       Type b = int is not compatible with type int
+       This instance of int is ambiguous:
+       it would escape the scope of its equation
+|}];;
+
+let f : type a b. (a,b) eq -> (b,int) eq -> a -> b -> _ = fun ab bint a b ->
+  let Eq = bint in
+  let x =
+    let Eq = ab in
+    if true then a else b
+  in ignore x
+;; (* ok *)
+[%%expect{|
+Line 5, characters 24-25:
+5 |     if true then a else b
+                            ^
+Error: This expression has type b = int
+       but an expression was expected of type a = int
+       Type int is not compatible with type a = int
+       This instance of int is ambiguous:
+       it would escape the scope of its equation
+|}];;
+
+let f (type a b c) (b : bool) (w1 : (a,b) eq) (w2 : (a,int) eq) (x : a) (y : b) =
+  let Eq = w1 in
+  let Eq = w2 in
+  if b then x else y
+;;
+[%%expect{|
+Line 4, characters 19-20:
+4 |   if b then x else y
+                       ^
+Error: This expression has type b = int
+       but an expression was expected of type a = int
+       Type a = int is not compatible with type a = int
+       This instance of int is ambiguous:
+       it would escape the scope of its equation
+|}];;
+
+let f (type a b c) (b : bool) (w1 : (a,b) eq) (w2 : (a,int) eq) (x : a) (y : b) =
+  let Eq = w1 in
+  let Eq = w2 in
+  if b then y else x
+[%%expect{|
+Line 4, characters 19-20:
+4 |   if b then y else x
+                       ^
+Error: This expression has type a = int
+       but an expression was expected of type b = int
+       This instance of int is ambiguous:
+       it would escape the scope of its equation
+|}];;
