@@ -1,3 +1,10 @@
+(* TEST
+flags = " -w a "
+* setup-ocamlc.byte-build-env
+** ocamlc.byte
+*** check-ocamlc.byte-output
+*)
+
 module X = struct module Y = struct module type S = sig type t end end end
 
 (* open X  (* works! *) *)
