@@ -1,3 +1,7 @@
+(* TEST
+   * expect
+*)
+
 (* Using generative functors *)
 
 (* Without type *)
@@ -25,7 +29,9 @@ module F : functor () -> S
 |}];;
 module G (X : sig end) : S = F ();; (* fail *)
 [%%expect{|
-Line _, characters 29-33:
+Line 1, characters 29-33:
+1 | module G (X : sig end) : S = F ();; (* fail *)
+                                 ^^^^
 Error: This expression creates fresh types.
        It is not allowed inside applicative functors.
 |}];;
@@ -43,7 +49,9 @@ module M : S
 |}];;
 module M = F(U);; (* fail *)
 [%%expect{|
-Line _, characters 11-12:
+Line 1, characters 11-12:
+1 | module M = F(U);; (* fail *)
+               ^
 Error: This is a generative functor. It can only be applied to ()
 |}];;
 
@@ -52,7 +60,9 @@ module F1 (X : sig end) = struct end;;
 module F2 : functor () -> sig end = F1;; (* fail *)
 [%%expect{|
 module F1 : functor (X : sig  end) -> sig  end
-Line _, characters 36-38:
+Line 2, characters 36-38:
+2 | module F2 : functor () -> sig end = F1;; (* fail *)
+                                        ^^
 Error: Signature mismatch:
        Modules do not match:
          functor (X : sig  end) -> sig  end
@@ -63,7 +73,9 @@ module F3 () = struct end;;
 module F4 : functor (X : sig end) -> sig end = F3;; (* fail *)
 [%%expect{|
 module F3 : functor () -> sig  end
-Line _, characters 47-49:
+Line 2, characters 47-49:
+2 | module F4 : functor (X : sig end) -> sig end = F3;; (* fail *)
+                                                   ^^
 Error: Signature mismatch:
        Modules do not match:
          functor () -> sig  end
