@@ -1,7 +1,14 @@
+(* TEST
+flags = " -w a "
+* setup-ocamlc.byte-build-env
+** ocamlc.byte
+*** check-ocamlc.byte-output
+*)
+
 module Classdef = struct
   class virtual ['a, 'b, 'c] cl0 =
-    object 
-      constraint 'c = < m : 'a -> 'b -> int; .. > 
+    object
+      constraint 'c = < m : 'a -> 'b -> int; .. >
     end
 
   class virtual ['a, 'b] cl1 =
