@@ -1,3 +1,7 @@
+(* TEST
+   * expect
+*)
+
 (*
    Polymorphic methods are now available in the main branch.
    Enjoy.
@@ -17,6 +21,75 @@ val f : 'a list -> 'a fold = <fun>
 - : int = 6
 |}];;
 
+type pty = {pv : 'a. 'a list};;
+[%%expect {|
+type pty = { pv : 'a. 'a list; }
+|}];;
+
+
+type id = { id : 'a. 'a -> 'a };;
+let id x = x;;
+let {id} = id { id };;
+[%%expect {|
+type id = { id : 'a. 'a -> 'a; }
+val id : 'a -> 'a = <fun>
+val id : 'a -> 'a = <fun>
+|}];;
+
+let px = {pv = []};;
+[%%expect {|
+val px : pty = {pv = []}
+|}];;
+
+match px with
+| {pv=[]} -> "OK"
+| {pv=5::_} -> "int"
+| {pv=true::_} -> "bool"
+;;
+[%%expect {|
+Line 1, characters 0-77:
+1 | match px with
+2 | | {pv=[]} -> "OK"
+3 | | {pv=5::_} -> "int"
+4 | | {pv=true::_} -> "bool"
+Warning 8: this pattern-matching is not exhaustive.
+Here is an example of a case that is not matched:
+{pv=false::_}
+- : string = "OK"
+|}];;
+
+match px with
+| {pv=[]} -> "OK"
+| {pv=true::_} -> "bool"
+| {pv=5::_} -> "int"
+;;
+[%%expect {|
+Line 1, characters 0-77:
+1 | match px with
+2 | | {pv=[]} -> "OK"
+3 | | {pv=true::_} -> "bool"
+4 | | {pv=5::_} -> "int"
+Warning 8: this pattern-matching is not exhaustive.
+Here is an example of a case that is not matched:
+{pv=0::_}
+- : string = "OK"
+|}];;
+
+match px with
+| {pv=[]} -> "OK"
+| {pv=5::_} -> "int"
+| {pv=true::_} -> "bool"
+| {pv=false::_} -> "bool"
+;;
+[%%expect {|
+- : string = "OK"
+|}];;
+
+fun {pv=v} -> true::v, 1::v;;
+[%%expect {|
+- : pty -> bool list * int list = <fun>
+|}];;
+
 class ['b] ilist l = object
   val l = l
   method add x = {< l = x :: l >}
@@ -228,7 +301,9 @@ class ['a] ostream1 :
     method tl : 'b
   end
 |}, Principal{|
-Line _, characters 4-16:
+Line 8, characters 4-16:
+8 |     self#tl#fold ~f ~init:(f self#hd init)
+        ^^^^^^^^^^^^
 Warning 18: this use of a polymorphic method is not principal.
 class ['a] ostream1 :
   hd:'a ->
@@ -385,9 +460,13 @@ val cp : color_point = <obj>
 val c : circle = <obj>
 val d : float = 11.
 val f : < m : 'a. 'a -> 'a > -> < m : 'b. 'b -> 'b > = <fun>
-Line _, characters 41-42:
+Line 9, characters 41-42:
+9 | let f (x : < m : 'a. 'a -> 'a list >) = (x : < m : 'b. 'b -> 'c >)
+                                             ^
 Error: This expression has type < m : 'b. 'b -> 'b list >
        but an expression was expected of type < m : 'b. 'b -> 'c >
+       The method m has type 'b. 'b -> 'b list,
+       but the expected method type was 'b. 'b -> 'c
        The universal variable 'b would escape its scope
 |}];;
 
@@ -435,7 +514,9 @@ class ['a] id1 = object
 end
 ;;
 [%%expect {|
-Line _, characters 12-17:
+Line 3, characters 12-17:
+3 |   method id x = x
+                ^^^^^
 Error: This method has type 'a -> 'a which is less general than 'b. 'b -> 'a
 |}];;
 
@@ -445,7 +526,9 @@ class id2 (x : 'a) = object
 end
 ;;
 [%%expect {|
-Line _, characters 12-17:
+Line 3, characters 12-17:
+3 |   method id x = x
+                ^^^^^
 Error: This method has type 'a -> 'a which is less general than 'b. 'b -> 'a
 |}];;
 
@@ -456,7 +539,9 @@ class id3 x = object
 end
 ;;
 [%%expect {|
-Line _, characters 12-17:
+Line 4, characters 12-17:
+4 |   method id _ = x
+                ^^^^^
 Error: This method has type 'b -> 'b which is less general than 'a. 'a -> 'a
 |}];;
 
@@ -470,7 +555,11 @@ class id4 () = object
 end
 ;;
 [%%expect {|
-Line _, characters 12-79:
+Line 4, characters 12-79:
+4 | ............x =
+5 |     match r with
+6 |       None -> r <- Some x; x
+7 |     | Some y -> y
 Error: This method has type 'b -> 'b which is less general than 'a. 'a -> 'a
 |}];;
 
@@ -495,7 +584,9 @@ let f4 f = ignore(f : id); f#id 1, f#id true
 [%%expect {|
 val f1 : id -> int * bool = <fun>
 val f2 : id -> int * bool = <fun>
-Line _, characters 24-28:
+Line 5, characters 24-28:
+5 | let f3 f = f#id 1, f#id true
+                            ^^^^
 Error: This expression has type bool but an expression was expected of type
          int
 |}];;
@@ -523,7 +614,9 @@ type 'a foo = 'a foo list
 [%%expect {|
 class id2 : object method id : 'a -> 'a method mono : int -> int end
 val app : int * bool = (1, true)
-Line _, characters 0-25:
+Line 9, characters 0-25:
+9 | type 'a foo = 'a foo list
+    ^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: The type abbreviation foo is cyclic
 |}];;
 
@@ -731,7 +824,9 @@ let bad2 = {bad2 = None};;
 bad2.bad2 <- Some (ref None);;
 [%%expect {|
 type bad = { bad : 'a. 'a option ref; }
-Line _, characters 17-25:
+Line 2, characters 17-25:
+2 | let bad = {bad = ref None};;
+                     ^^^^^^^^
 Error: This field value has type 'b option ref which is less general than
          'a. 'a option ref
 |}];;
@@ -784,7 +879,9 @@ object method virtual caseNil : 'a end
 and virtual int_list =
 object method virtual visit : 'a.('a visitor -> 'a) end;;
 [%%expect {|
-Line _, characters 30-51:
+Line 4, characters 30-51:
+4 | object method virtual visit : 'a.('a visitor -> 'a) end;;
+                                  ^^^^^^^^^^^^^^^^^^^^^
 Error: The universal type variable 'a cannot be generalized:
        it escapes its scope.
 |}];;
@@ -809,7 +906,9 @@ type t = { f : 'a 'b. ('b -> (#ct as 'a) -> 'b) -> 'b; }
 (* PR#1663 *)
 type t = u and u = t;;
 [%%expect {|
-Line _, characters 0-10:
+Line 1, characters 0-10:
+1 | type t = u and u = t;;
+    ^^^^^^^^^^
 Error: The definition of t contains a cycle:
        u
 |}];;
@@ -825,7 +924,9 @@ type t = [ `A of t a ]
 (* Wrong in 3.06 *)
 type ('a,'b) t constraint 'a = 'b and ('a,'b) u = ('a,'b) t;;
 [%%expect {|
-Line _, characters 50-59:
+Line 1, characters 50-59:
+1 | type ('a,'b) t constraint 'a = 'b and ('a,'b) u = ('a,'b) t;;
+                                                      ^^^^^^^^^
 Error: Constraints are not satisfied in this type.
        Type ('a, 'b) t should be an instance of ('c, 'c) t
 |}];;
@@ -843,7 +944,9 @@ type 'a u = 'a and 'a v = 'a u t;;
 type 'a u = 'a and 'a v = 'a u t constraint 'a = int;;
 [%%expect {|
 type 'a t constraint 'a = int
-Line _, characters 26-32:
+Line 2, characters 26-32:
+2 | type 'a u = 'a and 'a v = 'a u t;;
+                              ^^^^^^
 Error: Constraints are not satisfied in this type.
        Type 'a u t should be an instance of int t
 |}];;
@@ -856,7 +959,9 @@ type 'a u = 'a and 'a v = 'a u t constraint 'a = int;;
 [%%expect {|
 type g = int
 type 'a t = unit constraint 'a = g
-Line _, characters 26-32:
+Line 3, characters 26-32:
+3 | type 'a u = 'a and 'a v = 'a u t;;
+                              ^^^^^^
 Error: Constraints are not satisfied in this type.
        Type 'a u t should be an instance of g t
 |}];;
@@ -864,7 +969,9 @@ Error: Constraints are not satisfied in this type.
 (* Example of wrong expansion *)
 type 'a u = < m : 'a v > and 'a v = 'a list u;;
 [%%expect {|
-Line _, characters 0-24:
+Line 1, characters 0-24:
+1 | type 'a u = < m : 'a v > and 'a v = 'a list u;;
+    ^^^^^^^^^^^^^^^^^^^^^^^^
 Error: In the definition of v, type 'a list u should be 'a u
 |}];;
 
@@ -890,53 +997,13 @@ type 'a t = < a : 'a >
 type u = 'a t as 'a
 |}];;
 
-
-(* Variant tests *)
-type t = A | B;;
-function `A,_ -> 1 | _,A -> 2 | _,B -> 3;;
-function `A,_ -> 1 | _,(A|B) -> 2;;
-function Some `A, _ -> 1 | Some _, A -> 2 | None, A -> 3 | _, B -> 4;;
-function Some `A, A -> 1 | Some `A, B -> 1
-       | Some _, A -> 2  | None, A -> 3 | _, B -> 4;;
-function A, `A -> 1 | A, `B -> 2 | B, _ -> 3;;
-function `A, A -> 1 | `B, A -> 2 | _, B -> 3;;
-function (`A|`B), _ -> 0 | _,(`A|`B) -> 1;;
-function `B,1 -> 1 | _,1 -> 2;;
-function 1,`B -> 1 | 1,_ -> 2;;
-[%%expect {|
-type t = A | B
-- : [> `A ] * t -> int = <fun>
-- : [> `A ] * t -> int = <fun>
-- : [> `A ] option * t -> int = <fun>
-- : [> `A ] option * t -> int = <fun>
-- : t * [< `A | `B ] -> int = <fun>
-- : [< `A | `B ] * t -> int = <fun>
-Line _, characters 0-41:
-Warning 8: this pattern-matching is not exhaustive.
-Here is an example of a case that is not matched:
-(`AnyExtraTag, `AnyExtraTag)
-- : [> `A | `B ] * [> `A | `B ] -> int = <fun>
-Line _, characters 0-29:
-Warning 8: this pattern-matching is not exhaustive.
-Here is an example of a case that is not matched:
-(_, 0)
-Line _, characters 21-24:
-Warning 11: this match case is unused.
-- : [< `B ] * int -> int = <fun>
-Line _, characters 0-29:
-Warning 8: this pattern-matching is not exhaustive.
-Here is an example of a case that is not matched:
-(0, _)
-Line _, characters 21-24:
-Warning 11: this match case is unused.
-- : int * [< `B ] -> int = <fun>
-|}];;
-
 (* pass typetexp, but fails during Typedecl.check_recursion *)
 type ('a, 'b) a = 'a -> unit constraint 'a = [> `B of ('a, 'b) b as 'b]
 and  ('a, 'b) b = 'b -> unit constraint 'b = [> `A of ('a, 'b) a as 'a];;
 [%%expect {|
-Line _, characters 0-71:
+Line 1, characters 0-71:
+1 | type ('a, 'b) a = 'a -> unit constraint 'a = [> `B of ('a, 'b) b as 'b]
+    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: The definition of a contains a cycle:
        [> `B of ('a, 'b) b as 'b ] as 'a
 |}];;
@@ -1012,11 +1079,15 @@ end;;
 class c : object method m : int end
 val f : unit -> c = <fun>
 val f : unit -> c = <fun>
-Line _, characters 11-60:
+Line 4, characters 11-60:
+4 | let f () = object method private n = 1 method m = {<>}#n end;;
+               ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Warning 15: the following private methods were made public implicitly:
  n.
 val f : unit -> < m : int; n : int > = <fun>
-Line _, characters 11-56:
+Line 5, characters 11-56:
+5 | let f () = object (self:c) method n = 1 method m = 2 end;;
+               ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: This object is expected to have type c but actually has type
          < m : int; n : 'a >
        The first object type has no method n
@@ -1032,11 +1103,15 @@ type 'a bar = <m: 'b. 'a * <m: 'c. 'c * 'a bar> >
 type bar' =   <m: 'a. 'a * 'a bar >
 let f (x : foo') = (x : bar');;
 [%%expect {|
-Line _, characters 3-4:
+Line 2, characters 3-4:
+2 |   (x : <m : 'a. 'a * (<m:'b. 'a * <m:'c. 'c * 'bar> > as 'bar) >);;
+       ^
 Error: This expression has type < m : 'a. 'a * < m : 'a * 'b > > as 'b
        but an expression was expected of type
          < m : 'a. 'a * (< m : 'a * < m : 'c. 'c * 'd > > as 'd) >
-       Types for method m are incompatible
+       The method m has type 'a. 'a * 'd, but the expected method type was
+       'c. 'c * 'd
+       The universal variable 'a would escape its scope
 |}];;
 
 fun (x : <m : 'a. 'a * ('a * <m : 'a. 'a * 'foo> as 'foo)>) ->
@@ -1049,7 +1124,9 @@ let f x =
     (x : <m : 'a. 'a -> ('a * <m:'c. 'c -> 'bar> as 'bar)>
        :> <m : 'a. 'a -> ('a * 'foo)> as 'foo);;
 [%%expect {|
-Line _, characters 3-4:
+Line 2, characters 3-4:
+2 |   (x : <m : 'b. 'b * ('b * <m : 'c. 'c * ('c * 'bar)>)> as 'bar);;
+       ^
 Error: This expression has type
          < m : 'b. 'b * ('b * < m : 'c. 'c * 'a > as 'a) >
        but an expression was expected of type
@@ -1064,7 +1141,9 @@ module M
 : sig type t = <m : 'b. 'b * ('b * <m:'c. 'c * 'bar> as 'bar)> end
 = struct type t = <m : 'a. 'a * ('a * 'foo)> as 'foo end;;
 [%%expect {|
-Line _, characters 2-64:
+Line 3, characters 2-64:
+3 | = struct let f (x : <m : 'a. 'a * ('a * 'foo)> as 'foo) = () end;;
+      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: Signature mismatch:
        Modules do not match:
          sig val f : (< m : 'a. 'a * ('a * 'b) > as 'b) -> unit end
@@ -1122,7 +1201,9 @@ type v = private [> t ]
 - : t -> v = <fun>
 type u = private [< t ]
 - : u -> v = <fun>
-Line _, characters 9-21:
+Line 6, characters 9-21:
+6 | fun x -> (x : v :> u);;
+             ^^^^^^^^^^^^
 Error: Type v = [> `A | `B ] is not a subtype of u = [< `A | `B ]
 |}];;
 
@@ -1141,7 +1222,9 @@ let f5 x =
 let f6 x =
   (x : <m:'a. [< `A of < > ] as 'a> :> <m:'a. [< `A of <p:int> ] as 'a>);;
 [%%expect {|
-Line _, characters 2-88:
+Line 2, characters 2-88:
+2 | ..(x : <m:'a. (<p:int;..> as 'a) -> int>
+3 |     :> <m:'b. (<p:int;q:int;..> as 'b) -> int>)..
 Error: Type < m : 'a. (< p : int; .. > as 'a) -> int > is not a subtype of
          < m : 'b. (< p : int; q : int; .. > as 'b) -> int >
        Type < p : int; q : int; .. > as 'c is not a subtype of
@@ -1164,15 +1247,21 @@ val f : < m : 'a. 'a -> 'a > -> < m : 'a. 'a -> 'a > array = <fun>
 - : < m : 'a. 'a -> 'a > -> 'b -> 'b = <fun>
 |}, Principal{|
 val f : < m : 'a. 'a -> 'a > -> < m : 'a. 'a -> 'a > = <fun>
-Line _, characters 9-16:
+Line 2, characters 9-16:
+2 | fun x -> (f x)#m;; (* Warning 18 *)
+             ^^^^^^^
 Warning 18: this use of a polymorphic method is not principal.
 - : < m : 'a. 'a -> 'a > -> 'b -> 'b = <fun>
 val f : < m : 'a. 'a -> 'a > * 'b -> < m : 'a. 'a -> 'a > = <fun>
-Line _, characters 9-20:
+Line 4, characters 9-20:
+4 | fun x -> (f (x,x))#m;; (* Warning 18 *)
+             ^^^^^^^^^^^
 Warning 18: this use of a polymorphic method is not principal.
 - : < m : 'a. 'a -> 'a > -> 'b -> 'b = <fun>
 val f : < m : 'a. 'a -> 'a > -> < m : 'a. 'a -> 'a > array = <fun>
-Line _, characters 9-20:
+Line 6, characters 9-20:
+6 | fun x -> (f x).(0)#m;; (* Warning 18 *)
+             ^^^^^^^^^^^
 Warning 18: this use of a polymorphic method is not principal.
 - : < m : 'a. 'a -> 'a > -> 'b -> 'b = <fun>
 |}];;
@@ -1199,10 +1288,14 @@ val h : < id : 'a; .. > -> 'a = <fun>
 class c : object method id : 'a -> 'a end
 type u = c option
 val just : 'a option -> 'a = <fun>
-Line _, characters 42-62:
+Line 4, characters 42-62:
+4 | let f x = let l = [Some x; (None : u)] in (just(List.hd l))#id;;
+                                              ^^^^^^^^^^^^^^^^^^^^
 Warning 18: this use of a polymorphic method is not principal.
 val f : c -> 'a -> 'a = <fun>
-Line _, characters 36-47:
+Line 7, characters 36-47:
+7 |   let x = List.hd [Some x; none] in (just x)#id;;
+                                        ^^^^^^^^^^^
 Warning 18: this use of a polymorphic method is not principal.
 val g : c -> 'a -> 'a = <fun>
 val h : < id : 'a; .. > -> 'a = <fun>
@@ -1243,7 +1336,9 @@ val f : 'a -> int = <fun>
 val g : 'a -> int = <fun>
 type 'a t = Leaf of 'a | Node of ('a * 'a) t
 val depth : 'a t -> int = <fun>
-Line _, characters 2-42:
+Line 6, characters 2-42:
+6 |   function Leaf _ -> 1 | Node x -> 1 + d x
+      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: This definition has type 'a t -> int which is less general than
          'a0. 'a0 t -> int
 |}];;
@@ -1257,7 +1352,9 @@ let zero = {f = `Int 0} ;; (* fails *)
 type t = { f : 'a. [> `B of 'a | `Int of int ] as 'a; }
 val zero : t = {f = `Int 0}
 type t = { f : 'a. [< `Int of int ] as 'a; }
-Line _, characters 16-22:
+Line 4, characters 16-22:
+4 | let zero = {f = `Int 0} ;; (* fails *)
+                    ^^^^^^
 Error: This expression has type [> `Int of int ]
        but an expression was expected of type [< `Int of int ]
        Types for tag `Int are incompatible
@@ -1307,7 +1404,9 @@ let f ?x y = y in {f};; (* fail *)
 [%%expect {|
 type t = { f : 'a. 'a -> unit; }
 - : t = {f = <fun>}
-Line _, characters 19-20:
+Line 3, characters 19-20:
+3 | let f ?x y = y in {f};; (* fail *)
+                       ^
 Error: This field value has type unit -> unit which is less general than
          'a. 'a -> unit
 |}];;
@@ -1337,14 +1436,16 @@ let {foo} = (raise Exit : t);;
 type s = A of int
 let (A x) = (raise Exit : s);;
 [%%expect {|
-Exception: Pervasives.Exit.
+Exception: Stdlib.Exit.
 |}];;
 
 (* PR#5224 *)
 
 type 'x t = < f : 'y. 'y t >;;
 [%%expect {|
-Line _, characters 0-28:
+Line 1, characters 0-28:
+1 | type 'x t = < f : 'y. 'y t >;;
+    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: In the definition of t, type 'y t should be 'x t
 |}];;
 
@@ -1376,43 +1477,35 @@ end;;
 [%%expect {|
 val n : < m : 'x 'a. ([< `Foo of 'x ] as 'a) -> 'x > = <obj>
 |}];;
-(* ok, but not with -principal *)
+(* ok *)
 let n =
   object method m : 'x. [< `Foo of 'x] -> 'x = fun x -> assert false end;;
 [%%expect {|
 val n : < m : 'x. [< `Foo of 'x ] -> 'x > = <obj>
-|}, Principal{|
-Line _, characters 47-68:
-Error: This method has type ([< `Foo of 'b ] as 'a) -> 'b
-       which is less general than 'x. 'a -> 'x
 |}];;
 (* fail *)
 let (n : < m : 'a. [< `Foo of int] -> 'a >) =
   object method m : 'x. [< `Foo of 'x] -> 'x = fun x -> assert false end;;
 [%%expect {|
-Line _, characters 2-72:
+Line 2, characters 2-72:
+2 |   object method m : 'x. [< `Foo of 'x] -> 'x = fun x -> assert false end;;
+      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: This expression has type < m : 'x. [< `Foo of 'x ] -> 'x >
        but an expression was expected of type
          < m : 'a. [< `Foo of int ] -> 'a >
        The universal variable 'x would escape its scope
-|}, Principal{|
-Line _, characters 47-68:
-Error: This method has type ([< `Foo of 'b ] as 'a) -> 'b
-       which is less general than 'x. 'a -> 'x
 |}];;
 (* fail *)
 let (n : 'b -> < m : 'a . ([< `Foo of int] as 'b) -> 'a >) = fun x ->
   object method m : 'x. [< `Foo of 'x] -> 'x = fun x -> assert false end;;
 [%%expect {|
-Line _, characters 2-72:
+Line 2, characters 2-72:
+2 |   object method m : 'x. [< `Foo of 'x] -> 'x = fun x -> assert false end;;
+      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Error: This expression has type < m : 'x. [< `Foo of 'x ] -> 'x >
        but an expression was expected of type
          < m : 'a. [< `Foo of int ] -> 'a >
        The universal variable 'x would escape its scope
-|}, Principal{|
-Line _, characters 47-68:
-Error: This method has type ([< `Foo of 'b ] as 'a) -> 'b
-       which is less general than 'x. 'a -> 'x
 |}];;
 
 (* PR#6171 *)
@@ -1420,7 +1513,9 @@ let f b (x: 'x) =
   let module M = struct type t = A end in
   if b then x else M.A;;
 [%%expect {|
-Line _, characters 19-22:
+Line 3, characters 19-22:
+3 |   if b then x else M.A;;
+                       ^^^
 Error: This expression has type M.t but an expression was expected of type 'x
        The type constructor M.t would escape its scope
 |}];;
@@ -1512,16 +1607,19 @@ type h = < m : int; n : string; x : string; y : int >
 type t = <g>
 and g = <a:t>
 [%%expect{|
-Line _, characters 10-11:
-Error: The type constructor g
-is not yet completely defined
+Line 1, characters 10-11:
+1 | type t = <g>
+              ^
+Error: The type constructor g is not yet completely defined
 |}]
 
 type t = int
 type g = <t>
 [%%expect{|
 type t = int
-Line _, characters 10-11:
+Line 2, characters 10-11:
+2 | type g = <t>
+              ^
 Error: The type int is not an object type
 |}]
 
@@ -1560,7 +1658,9 @@ type r2 = < a : int >
 
 type gg = <a:int->float; a:int>
 [%%expect{|
-Line _, characters 27-30:
+Line 1, characters 27-30:
+1 | type gg = <a:int->float; a:int>
+                               ^^^
 Error: Method 'a' has type int, which should be int -> float
 |}]
 
@@ -1568,7 +1668,9 @@ type t = <a:int; b:string>
 type g = <b:float; t;>
 [%%expect{|
 type t = < a : int; b : string >
-Line _, characters 19-20:
+Line 2, characters 19-20:
+2 | type g = <b:float; t;>
+                       ^
 Error: Method 'b' has type string, which should be float
 |}]
 
@@ -1583,7 +1685,9 @@ type t = < f : int >
 
 type t = < int #A.t1 >
 [%%expect{|
-Line _, characters 11-20:
+Line 1, characters 11-20:
+1 | type t = < int #A.t1 >
+               ^^^^^^^^^
 Error: Illegal open object type
 |}]
 
@@ -1598,11 +1702,35 @@ type 'a t = < m : 'a > constraint 'a = int
 |}]
 
 (* GPR#1142 *)
+external reraise : exn -> 'a = "%reraise"
+
 module M () = struct
   let f : 'a -> 'a = assert false
   let g : 'a -> 'a = raise Not_found
+  let h : 'a -> 'a = reraise Not_found
+  let i : 'a -> 'a = raise_notrace Not_found
 end
 
 [%%expect{|
-module M : functor () -> sig val f : 'a -> 'a val g : 'a -> 'a end
+external reraise : exn -> 'a = "%reraise"
+module M :
+  functor () ->
+    sig
+      val f : 'a -> 'a
+      val g : 'a -> 'a
+      val h : 'a -> 'a
+      val i : 'a -> 'a
+    end
+|}]
+
+(* #8550 *)
+class ['a] r = let r : 'a = ref [] in object method get = r end;;
+[%%expect{|
+Line 1, characters 0-63:
+1 | class ['a] r = let r : 'a = ref [] in object method get = r end;;
+    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+Error: The type of this class,
+       class ['a] r :
+         object constraint 'a = '_weak2 list ref method get : 'a end,
+       contains type variables that cannot be generalized
 |}]
