@@ -1,3 +1,10 @@
+(* TEST
+flags = " -w a "
+* setup-ocamlc.byte-build-env
+** ocamlc.byte
+*** check-ocamlc.byte-output
+*)
+
 (* OK *)
 class type [ 'node ] extension = object method node : 'node end
 class type [ 'ext ] node = object constraint 'ext = 'ext node #extension end
