@@ -1,3 +1,13 @@
+(* TEST
+
+flags = "-w A"
+
+* setup-ocamlc.byte-build-env
+** ocamlc.byte
+compile_only = "true"
+*** check-ocamlc.byte-output
+
+*)
 
 let f = (fun x -> x) [@inline] [@inline never]
 let g = (fun x -> x) [@inline] [@something_else] [@ocaml.inline]
