@@ -1,3 +1,14 @@
+(* TEST
+
+flags = "-w A"
+
+* setup-ocamlc.byte-build-env
+** ocamlc.byte
+compile_only = "true"
+*** check-ocamlc.byte-output
+
+*)
+
 (* PR#7314 *)
 
 module type Comparable = sig
