@@ -23,6 +23,9 @@ let option opt () = compargs := opt :: !compargs
 let option_with_arg opt arg =
   compargs := (Filename.quote arg) :: opt :: !compargs
 ;;
+let option_with_int opt arg =
+  compargs := (Int.to_string arg) :: opt :: !compargs
+;;
 
 let make_archive = ref false;;
 let with_impl = ref false;;
@@ -45,6 +48,7 @@ let incompatible o =
 module Options = Main_args.Make_bytecomp_options (struct
   let _a () = make_archive := true; option "-a" ()
   let _absname = option "-absname"
+  let _alert = option_with_arg "-alert"
   let _annot = option "-annot"
   let _binannot = option "-bin-annot"
   let _c = option "-c"
@@ -52,6 +56,7 @@ module Options = Main_args.Make_bytecomp_options (struct
   let _cclib s = option_with_arg "-cclib" s
   let _ccopt s = option_with_arg "-ccopt" s
   let _config = option "-config"
+  let _config_var s = option_with_arg "-config-var" s
   let _compat_32 = option "-compat-32"
   let _custom = option "-custom"
   let _dllib = option_with_arg "-dllib"
@@ -59,6 +64,7 @@ module Options = Main_args.Make_bytecomp_options (struct
   let _dtypes = option "-dtypes"
   let _for_pack = option_with_arg "-for-pack"
   let _g = option "-g"
+  let _stop_after = option_with_arg "-stop-after"
   let _i = option "-i"
   let _I s = option_with_arg "-I" s
   let _impl s = with_impl := true; option_with_arg "-impl" s
@@ -116,8 +122,13 @@ module Options = Main_args.Make_bytecomp_options (struct
   let _warn_error = option_with_arg "-warn-error"
   let _warn_help = option "-warn-help"
   let _color s = option_with_arg "-color" s
+  let _error_style s = option_with_arg "-error-style" s
   let _where = option "-where"
   let _nopervasives = option "-nopervasives"
+  let _match_context_rows n = option_with_int "-match-context-rows" n
+  let _dump_into_file = option "-dump-into-file"
+  let _dno_unique_ids = option "-dno-unique-ids"
+  let _dunique_ids = option "-dunique-ids"
   let _dsource = option "-dsource"
   let _dparsetree = option "-dparsetree"
   let _dtypedtree = option "-dtypedtree"
@@ -125,6 +136,7 @@ module Options = Main_args.Make_bytecomp_options (struct
   let _dlambda = option "-dlambda"
   let _dflambda = option "-dflambda"
   let _dinstr = option "-dinstr"
+  let _dcamlprimc = option "-dcamlprimc"
   let _dtimings = option "-dtimings"
   let _dprofile = option "-dprofile"
   let _args = Arg.read_arg
