@@ -24,7 +24,7 @@ let option_with_arg opt arg =
   compargs := (Filename.quote arg) :: opt :: !compargs
 ;;
 let option_with_int opt arg =
-  compargs := (string_of_int arg) :: opt :: !compargs
+  compargs := (Int.to_string arg) :: opt :: !compargs
 ;;
 let option_with_float opt arg =
   compargs := (string_of_float arg) :: opt :: !compargs
@@ -53,6 +53,7 @@ module Options = Main_args.Make_optcomp_options (struct
   let _absname = option "-absname"
   let _afl_instrument = option "-afl-instrument"
   let _afl_inst_ratio n = option_with_int "-afl-inst-ratio" n
+  let _alert = option_with_arg "-alert"
   let _annot = option "-annot"
   let _binannot = option "-bin-annot"
   let _c = option "-c"
@@ -62,8 +63,10 @@ module Options = Main_args.Make_optcomp_options (struct
   let _clambda_checks = option "-clambda-checks"
   let _compact = option "-compact"
   let _config = option "-config"
+  let _config_var s = option_with_arg "-config-var" s
   let _for_pack s = option_with_arg "-for-pack" s
   let _g = option "-g"
+  let _stop_after = option_with_arg "-stop-after"
   let _i = option "-i"
   let _I s = option_with_arg "-I" s
   let _impl s = with_impl := true; option_with_arg "-impl" s
@@ -73,7 +76,7 @@ module Options = Main_args.Make_optcomp_options (struct
   let _dump_pass = option_with_arg "-dump-pass"
   let _inline_max_depth n = option_with_arg "-inline-max-depth" n
   let _rounds n = option_with_int "-rounds" n
-  let _inline_max_unroll n = option_with_arg "-unroll" n
+  let _inline_max_unroll n = option_with_arg "-inline-max-unroll" n
   let _inline_call_cost n = option_with_arg "-inline-call-cost" n
   let _inline_alloc_cost n = option_with_arg "-inline-alloc-cost" n
   let _inline_prim_cost n = option_with_arg "-inline-prim-cost" n
@@ -142,10 +145,15 @@ module Options = Main_args.Make_optcomp_options (struct
   let _warn_error = option_with_arg "-warn-error"
   let _warn_help = option "-warn-help"
   let _color s = option_with_arg "-color" s
+  let _error_style s = option_with_arg "-error-style" s
   let _where = option "-where"
 
   let _linscan = option "-linscan"
   let _nopervasives = option "-nopervasives"
+  let _match_context_rows n = option_with_int "-match-context-rows" n
+  let _dump_into_file = option "-dump-into-file"
+  let _dno_unique_ids = option "-dno-unique_ids"
+  let _dunique_ids = option "-dunique_ids"
   let _dsource = option "-dsource"
   let _dparsetree = option "-dparsetree"
   let _dtypedtree = option "-dtypedtree"
@@ -155,6 +163,7 @@ module Options = Main_args.Make_optcomp_options (struct
   let _dclambda = option "-dclambda"
   let _drawflambda = option "-drawflambda"
   let _dflambda = option "-dflambda"
+  let _dflambda_invariants = option "-dflambda-invariants"
   let _dflambda_no_invariants = option "-dflambda-no-invariants"
   let _dflambda_let stamp = option_with_int "-dflambda-let" stamp
   let _dflambda_verbose = option "-dflambda-verbose"
