@@ -24,7 +24,15 @@ open Typedtree
 
 exception Dont_match
 
-let value_descriptions env vd1 vd2 =
+let value_descriptions ~loc env name
+    (vd1 : Types.value_description)
+    (vd2 : Types.value_description) =
+  Builtin_attributes.check_alerts_inclusion
+    ~def:vd1.val_loc
+    ~use:vd2.val_loc
+    loc
+    vd1.val_attributes vd2.val_attributes
+    name;
   if Ctype.moregeneral env true vd1.val_type vd2.val_type then begin
     match (vd1.val_kind, vd2.val_kind) with
         (Val_prim p1, Val_prim p2) ->
@@ -161,122 +169,153 @@ let report_type_mismatch0 first second decl ppf err =
          "uses unboxed representation"
   | Immediate -> pr "%s is not an immediate type" first
 
-let report_type_mismatch first second decl ppf =
-  List.iter
-    (fun err ->
-      if err = Manifest then () else
-      Format.fprintf ppf "@ %a." (report_type_mismatch0 first second decl) err)
+let report_type_mismatch first second decl ppf err =
+  if err = Manifest then () else
+  Format.fprintf ppf "@ %a." (report_type_mismatch0 first second decl) err
 
-let rec compare_constructor_arguments env cstr params1 params2 arg1 arg2 =
+let rec compare_constructor_arguments ~loc env cstr params1 params2 arg1 arg2 =
   match arg1, arg2 with
   | Types.Cstr_tuple arg1, Types.Cstr_tuple arg2 ->
-      if List.length arg1 <> List.length arg2 then [Field_arity cstr]
+      if List.length arg1 <> List.length arg2 then Some (Field_arity cstr)
       else if
         (* Ctype.equal must be called on all arguments at once, cf. PR#7378 *)
         Ctype.equal env true (params1 @ arg1) (params2 @ arg2)
-      then [] else [Field_type cstr]
+      then None else Some (Field_type cstr)
   | Types.Cstr_record l1, Types.Cstr_record l2 ->
-      compare_records env params1 params2 0 l1 l2
-  | _ -> [Field_type cstr]
+      compare_records env ~loc params1 params2 0 l1 l2
+  | _ -> Some (Field_type cstr)
 
-and compare_variants env params1 params2 n cstrs1 cstrs2 =
+and compare_variants ~loc env params1 params2 n
+    (cstrs1 : Types.constructor_declaration list)
+    (cstrs2 : Types.constructor_declaration list) =
   match cstrs1, cstrs2 with
-    [], []           -> []
-  | [], c::_ -> [Field_missing (true, c.Types.cd_id)]
-  | c::_, [] -> [Field_missing (false, c.Types.cd_id)]
-  | {Types.cd_id=cstr1; cd_args=arg1; cd_res=ret1}::rem1,
-    {Types.cd_id=cstr2; cd_args=arg2; cd_res=ret2}::rem2 ->
-      if Ident.name cstr1 <> Ident.name cstr2 then
-        [Field_names (n, cstr1, cstr2)]
-      else
+    [], []           -> None
+  | [], c::_ -> Some (Field_missing (true, c.Types.cd_id))
+  | c::_, [] -> Some (Field_missing (false, c.Types.cd_id))
+  | cd1::rem1, cd2::rem2 ->
+      if Ident.name cd1.cd_id <> Ident.name cd2.cd_id then
+        Some (Field_names (n, cd1.cd_id, cd2.cd_id))
+      else begin
+        Builtin_attributes.check_alerts_inclusion
+          ~def:cd1.cd_loc
+          ~use:cd2.cd_loc
+          loc
+          cd1.cd_attributes cd2.cd_attributes
+          (Ident.name cd1.cd_id);
         let r =
-          match ret1, ret2 with
+          match cd1.cd_res, cd2.cd_res with
           | Some r1, Some r2 ->
               if Ctype.equal env true [r1] [r2] then
-                compare_constructor_arguments env cstr1 [r1] [r2] arg1 arg2
-              else [Field_type cstr1]
+                compare_constructor_arguments ~loc env cd1.cd_id [r1] [r2]
+                  cd1.cd_args cd2.cd_args
+              else Some (Field_type cd1.cd_id)
           | Some _, None | None, Some _ ->
-              [Field_type cstr1]
+              Some (Field_type cd1.cd_id)
           | _ ->
-              compare_constructor_arguments env cstr1
-                params1 params2 arg1 arg2
+              compare_constructor_arguments ~loc env cd1.cd_id
+                params1 params2 cd1.cd_args cd2.cd_args
         in
-        if r <> [] then r
-        else compare_variants env params1 params2 (n+1) rem1 rem2
+        if r <> None then r
+        else compare_variants ~loc env params1 params2 (n+1) rem1 rem2
+      end
 
 
-and compare_records env params1 params2 n labels1 labels2 =
+and compare_records ~loc env params1 params2 n
+    (labels1 : Types.label_declaration list)
+    (labels2 : Types.label_declaration list) =
   match labels1, labels2 with
-    [], []           -> []
-  | [], l::_ -> [Field_missing (true, l.Types.ld_id)]
-  | l::_, [] -> [Field_missing (false, l.Types.ld_id)]
-  | {Types.ld_id=lab1; ld_mutable=mut1; ld_type=arg1}::rem1,
-    {Types.ld_id=lab2; ld_mutable=mut2; ld_type=arg2}::rem2 ->
-      if Ident.name lab1 <> Ident.name lab2
-      then [Field_names (n, lab1, lab2)]
-      else if mut1 <> mut2 then [Field_mutable lab1] else
-      if Ctype.equal env true (arg1::params1)
-                              (arg2::params2)
-      then (* add arguments to the parameters, cf. PR#7378 *)
-        compare_records env (arg1::params1) (arg2::params2) (n+1) rem1 rem2
-      else [Field_type lab1]
+    [], []           -> None
+  | [], l::_ -> Some (Field_missing (true, l.Types.ld_id))
+  | l::_, [] -> Some (Field_missing (false, l.Types.ld_id))
+  | ld1::rem1, ld2::rem2 ->
+      if Ident.name ld1.ld_id <> Ident.name ld2.ld_id
+      then Some (Field_names (n, ld1.ld_id, ld2.ld_id))
+      else if ld1.ld_mutable <> ld2.ld_mutable then
+        Some (Field_mutable ld1.ld_id)
+      else begin
+        Builtin_attributes.check_deprecated_mutable_inclusion
+          ~def:ld1.ld_loc
+          ~use:ld2.ld_loc
+          loc
+          ld1.ld_attributes ld2.ld_attributes
+          (Ident.name ld1.ld_id);
+        if Ctype.equal env true (ld1.ld_type::params1)(ld2.ld_type::params2)
+        then (* add arguments to the parameters, cf. PR#7378 *)
+          compare_records ~loc env
+            (ld1.ld_type::params1) (ld2.ld_type::params2)
+            (n+1)
+            rem1 rem2
+        else
+          Some (Field_type ld1.ld_id)
+      end
 
-let type_declarations ?(equality = false) env name decl1 id decl2 =
-  if decl1.type_arity <> decl2.type_arity then [Arity] else
-  if not (private_flags decl1 decl2) then [Privacy] else
+let type_declarations ?(equality = false) ~loc env ~mark name decl1 path decl2 =
+  Builtin_attributes.check_alerts_inclusion
+    ~def:decl1.type_loc
+    ~use:decl2.type_loc
+    loc
+    decl1.type_attributes decl2.type_attributes
+    name;
+  if decl1.type_arity <> decl2.type_arity then Some Arity else
+  if not (private_flags decl1 decl2) then Some Privacy else
   let err = match (decl1.type_manifest, decl2.type_manifest) with
       (_, None) ->
         if Ctype.equal env true decl1.type_params decl2.type_params
-        then [] else [Constraint]
+        then None else Some Constraint
     | (Some ty1, Some ty2) ->
         if type_manifest env ty1 decl1.type_params ty2 decl2.type_params
             decl2.type_private
-        then [] else [Manifest]
+        then None else Some Manifest
     | (None, Some ty2) ->
         let ty1 =
-          Btype.newgenty (Tconstr(Pident id, decl2.type_params, ref Mnil))
+          Btype.newgenty (Tconstr(path, decl2.type_params, ref Mnil))
         in
         if Ctype.equal env true decl1.type_params decl2.type_params then
-          if Ctype.equal env false [ty1] [ty2] then []
-          else [Manifest]
-        else [Constraint]
+          if Ctype.equal env false [ty1] [ty2] then None
+          else Some Manifest
+        else Some Constraint
   in
-  if err <> [] then err else
+  if err <> None then err else
   let err =
     match (decl2.type_kind, decl1.type_unboxed.unboxed,
            decl2.type_unboxed.unboxed) with
-    | Type_abstract, _, _ -> []
-    | _, true, false -> [Unboxed_representation false]
-    | _, false, true -> [Unboxed_representation true]
-    | _ -> []
+    | Type_abstract, _, _ -> None
+    | _, true, false -> Some (Unboxed_representation false)
+    | _, false, true -> Some (Unboxed_representation true)
+    | _ -> None
   in
-  if err <> [] then err else
+  if err <> None then err else
   let err = match (decl1.type_kind, decl2.type_kind) with
-      (_, Type_abstract) -> []
+      (_, Type_abstract) -> None
     | (Type_variant cstrs1, Type_variant cstrs2) ->
-        let mark cstrs usage name decl =
-          List.iter
-            (fun c ->
-              Env.mark_constructor_used usage env name decl
-                                        (Ident.name c.Types.cd_id))
-            cstrs
-        in
-        let usage =
-          if decl1.type_private = Private || decl2.type_private = Public
-          then Env.Positive else Env.Privatize
-        in
-        mark cstrs1 usage name decl1;
-        if equality then mark cstrs2 Env.Positive (Ident.name id) decl2;
-        compare_variants env decl1.type_params decl2.type_params 1 cstrs1 cstrs2
+        if mark then begin
+          let mark cstrs usage name decl =
+            List.iter
+              (fun c ->
+                 Env.mark_constructor_used usage name decl
+                   (Ident.name c.Types.cd_id))
+              cstrs
+          in
+          let usage =
+            if decl1.type_private = Private || decl2.type_private = Public
+            then Env.Positive else Env.Privatize
+          in
+          mark cstrs1 usage name decl1;
+          if equality then mark cstrs2 Env.Positive (Path.name path) decl2
+        end;
+        compare_variants ~loc env decl1.type_params
+          decl2.type_params 1 cstrs1 cstrs2
     | (Type_record(labels1,rep1), Type_record(labels2,rep2)) ->
-        let err = compare_records env decl1.type_params decl2.type_params
-            1 labels1 labels2 in
-        if err <> [] || rep1 = rep2 then err else
-        [Record_representation (rep2 = Record_float)]
-    | (Type_open, Type_open) -> []
-    | (_, _) -> [Kind]
+        let err =
+          compare_records ~loc env decl1.type_params
+            decl2.type_params 1 labels1 labels2
+        in
+        if err <> None || rep1 = rep2 then err else
+        Some (Record_representation (rep2 = Record_float))
+    | (Type_open, Type_open) -> None
+    | (_, _) -> Some Kind
   in
-  if err <> [] then err else
+  if err <> None then err else
   let abstr = decl2.type_kind = Type_abstract && decl2.type_manifest = None in
   (* If attempt to assign a non-immediate type (e.g. string) to a type that
    * must be immediate, then we error *)
@@ -284,10 +323,13 @@ let type_declarations ?(equality = false) env name decl1 id decl2 =
     if abstr &&
        not decl1.type_immediate &&
        decl2.type_immediate then
-      [Immediate]
-    else []
+      Some Immediate
+    else None
   in
-  if err <> [] then err else
+  if err <> None then err else
+  let need_variance =
+    abstr || decl1.type_private = Private || decl1.type_kind = Type_open in
+  if not need_variance then None else
   let abstr = abstr || decl2.type_private = Private in
   let opn = decl2.type_kind = Type_open && decl2.type_manifest = None in
   let constrained ty = not (Btype.(is_Tvar (repr ty))) in
@@ -302,37 +344,40 @@ let type_declarations ?(equality = false) env name decl1 id decl2 =
         let (p1,n1,i1,j1) = get_lower v1 and (p2,n2,i2,j2) = get_lower v2 in
         imp abstr (imp p2 p1 && imp n2 n1 && imp i2 i1 && imp j2 j1))
       decl2.type_params (List.combine decl1.type_variance decl2.type_variance)
-  then [] else [Variance]
+  then None else Some Variance
 
 (* Inclusion between extension constructors *)
 
-let extension_constructors env id ext1 ext2 =
-  let usage =
-    if ext1.ext_private = Private || ext2.ext_private = Public
-    then Env.Positive else Env.Privatize
-  in
-  Env.mark_extension_used usage env ext1 (Ident.name id);
+let extension_constructors ~loc env ~mark id ext1 ext2 =
+  if mark then begin
+    let usage =
+      if ext1.ext_private = Private || ext2.ext_private = Public
+      then Env.Positive else Env.Privatize
+    in
+    Env.mark_extension_used usage ext1 (Ident.name id)
+  end;
   let ty1 =
     Btype.newgenty (Tconstr(ext1.ext_type_path, ext1.ext_type_params, ref Mnil))
   in
   let ty2 =
     Btype.newgenty (Tconstr(ext2.ext_type_path, ext2.ext_type_params, ref Mnil))
   in
-  if Ctype.equal env true
-       (ty1 :: ext1.ext_type_params)
-       (ty2 :: ext2.ext_type_params)
-  then
-    if compare_constructor_arguments env (Ident.create "")
+  if not (Ctype.equal env true (ty1 :: ext1.ext_type_params)
+                               (ty2 :: ext2.ext_type_params))
+  then Some (Field_type id)
+  else
+    let r =
+      compare_constructor_arguments ~loc env id
         ext1.ext_type_params ext2.ext_type_params
-        ext1.ext_args ext2.ext_args = [] then
-      if match ext1.ext_ret_type, ext2.ext_ret_type with
-          Some r1, Some r2 when not (Ctype.equal env true [r1] [r2]) -> false
-        | Some _, None | None, Some _ -> false
-        | _ -> true
-      then
+        ext1.ext_args ext2.ext_args
+    in
+    if r <> None then r else
+    match ext1.ext_ret_type, ext2.ext_ret_type with
+      Some r1, Some r2 when not (Ctype.equal env true [r1] [r2]) ->
+        Some (Field_type id)
+    | Some _, None | None, Some _ ->
+        Some (Field_type id)
+    | _ ->
         match ext1.ext_private, ext2.ext_private with
-            Private, Public -> false
-          | _, _ -> true
-      else false
-    else false
-  else false
+          Private, Public -> Some Privacy
+        | _, _ -> None
