@@ -17,10 +17,11 @@ open Asttypes
 open Typedtree
 
 (* TODO: add 'methods' for location, attribute, extension,
-   open_description, include_declaration, include_description *)
+   include_declaration, include_description *)
 
 type mapper =
   {
+    binding_op: mapper -> binding_op -> binding_op;
     case: mapper -> case -> case;
     cases: mapper -> case list -> case list;
     class_declaration: mapper -> class_declaration -> class_declaration;
@@ -40,6 +41,7 @@ type mapper =
     module_binding: mapper -> module_binding -> module_binding;
     module_coercion: mapper -> module_coercion -> module_coercion;
     module_declaration: mapper -> module_declaration -> module_declaration;
+    module_substitution: mapper -> module_substitution -> module_substitution;
     module_expr: mapper -> module_expr -> module_expr;
     module_type: mapper -> module_type -> module_type;
     module_type_declaration:
@@ -47,15 +49,19 @@ type mapper =
     package_type: mapper -> package_type -> package_type;
     pat: mapper -> pattern -> pattern;
     row_field: mapper -> row_field -> row_field;
+    object_field: mapper -> object_field -> object_field;
+    open_declaration: mapper -> open_declaration -> open_declaration;
+    open_description: mapper -> open_description -> open_description;
     signature: mapper -> signature -> signature;
     signature_item: mapper -> signature_item -> signature_item;
     structure: mapper -> structure -> structure;
     structure_item: mapper -> structure_item -> structure_item;
     typ: mapper -> core_type -> core_type;
     type_declaration: mapper -> type_declaration -> type_declaration;
-    type_declarations: mapper -> (rec_flag * type_declaration list) ->
-      (rec_flag * type_declaration list);
+    type_declarations: mapper -> (rec_flag * type_declaration list)
+      -> (rec_flag * type_declaration list);
     type_extension: mapper -> type_extension -> type_extension;
+    type_exception: mapper -> type_exception -> type_exception;
     type_kind: mapper -> type_kind -> type_kind;
     value_binding: mapper -> value_binding -> value_binding;
     value_bindings: mapper -> (rec_flag * value_binding list) ->
@@ -69,6 +75,7 @@ let tuple2 f1 f2 (x, y) = (f1 x, f2 y)
 let tuple3 f1 f2 f3 (x, y, z) = (f1 x, f2 y, f3 z)
 let opt f = function None -> None | Some x -> Some (f x)
 
+
 let structure sub {str_items; str_type; str_final_env} =
   {
     str_items = List.map (sub.structure_item sub) str_items;
@@ -90,6 +97,8 @@ let module_declaration sub x =
   let md_type = sub.module_type sub x.md_type in
   {x with md_type}
 
+let module_substitution _ x = x
+
 let include_infos f x = {x with incl_mod = f x.incl_mod}
 
 let class_type_declaration sub x =
@@ -111,7 +120,7 @@ let structure_item sub {str_desc; str_loc; str_env} =
         let (rec_flag, list) = sub.type_declarations sub (rec_flag, list) in
         Tstr_type (rec_flag, list)
     | Tstr_typext te -> Tstr_typext (sub.type_extension sub te)
-    | Tstr_exception ext -> Tstr_exception (sub.extension_constructor sub ext)
+    | Tstr_exception ext -> Tstr_exception (sub.type_exception sub ext)
     | Tstr_module mb -> Tstr_module (sub.module_binding sub mb)
     | Tstr_recmodule list ->
         Tstr_recmodule (List.map (sub.module_binding sub) list)
@@ -124,7 +133,7 @@ let structure_item sub {str_desc; str_loc; str_env} =
           (List.map (tuple3 id id (sub.class_type_declaration sub)) list)
     | Tstr_include incl ->
         Tstr_include (include_infos (sub.module_expr sub) incl)
-    | Tstr_open _
+    | Tstr_open od -> Tstr_open (sub.open_declaration sub od)
     | Tstr_attribute _ as d -> d
   in
   {str_desc; str_env; str_loc}
@@ -173,6 +182,12 @@ let type_extension sub x =
   in
   {x with tyext_constructors; tyext_params}
 
+let type_exception sub x =
+  let tyexn_constructor =
+    sub.extension_constructor sub x.tyexn_constructor
+  in
+  {x with tyexn_constructor}
+
 let extension_constructor sub x =
   let ext_kind =
     match x.ext_kind with
@@ -207,6 +222,7 @@ let pat sub x =
         Tpat_or (sub.pat sub p1, sub.pat sub p2, rd)
     | Tpat_alias (p, id, s) -> Tpat_alias (sub.pat sub p, id, s)
     | Tpat_lazy p -> Tpat_lazy (sub.pat sub p)
+    | Tpat_exception p -> Tpat_exception (sub.pat sub p)
   in
   {x with pat_extra; pat_desc; pat_env}
 
@@ -216,8 +232,6 @@ let expr sub x =
         Texp_constraint (sub.typ sub cty)
     | Texp_coerce (cty1, cty2) ->
         Texp_coerce (opt (sub.typ sub) cty1, sub.typ sub cty2)
-    | Texp_open (ovf, path, loc, env) ->
-        Texp_open (ovf, path, loc, sub.env sub env)
     | Texp_newtype _ as d -> d
     | Texp_poly cto -> Texp_poly (opt (sub.typ sub) cto)
   in
@@ -238,11 +252,10 @@ let expr sub x =
           sub.expr sub exp,
           List.map (tuple2 id (opt (sub.expr sub))) list
         )
-    | Texp_match (exp, cases, exn_cases, p) ->
+    | Texp_match (exp, cases, p) ->
         Texp_match (
           sub.expr sub exp,
           sub.cases sub cases,
-          sub.cases sub exn_cases,
           p
         )
     | Texp_try (exp, cases) ->
@@ -324,10 +337,11 @@ let expr sub x =
           path,
           List.map (tuple3 id id (sub.expr sub)) list
         )
-    | Texp_letmodule (id, s, mexpr, exp) ->
+    | Texp_letmodule (id, s, pres, mexpr, exp) ->
         Texp_letmodule (
           id,
           s,
+          pres,
           sub.module_expr sub mexpr,
           sub.expr sub exp
         )
@@ -344,10 +358,20 @@ let expr sub x =
         Texp_object (sub.class_structure sub cl, sl)
     | Texp_pack mexpr ->
         Texp_pack (sub.module_expr sub mexpr)
+    | Texp_letop {let_; ands; param; body; partial} ->
+        Texp_letop{
+          let_ = sub.binding_op sub let_;
+          ands = List.map (sub.binding_op sub) ands;
+          param;
+          body = sub.case sub body;
+          partial;
+        }
     | Texp_unreachable ->
         Texp_unreachable
     | Texp_extension_constructor _ as e ->
         e
+    | Texp_open (od, e) ->
+        Texp_open (sub.open_declaration sub od, sub.expr sub e)
   in
   {x with exp_extra; exp_desc; exp_env}
 
@@ -356,6 +380,9 @@ let package_type sub x =
   let pack_fields = List.map (tuple2 id (sub.typ sub)) x.pack_fields in
   {x with pack_fields}
 
+let binding_op sub x =
+  { x with bop_exp = sub.expr sub x.bop_exp }
+
 let signature sub x =
   let sig_final_env = sub.env sub x.sig_final_env in
   let sig_items = List.map (sub.signature_item sub) x.sig_items in
@@ -370,12 +397,17 @@ let signature_item sub x =
     | Tsig_type (rec_flag, list) ->
         let (rec_flag, list) = sub.type_declarations sub (rec_flag, list) in
         Tsig_type (rec_flag, list)
+    | Tsig_typesubst list ->
+        let (_, list) = sub.type_declarations sub (Nonrecursive, list) in
+        Tsig_typesubst list
     | Tsig_typext te ->
         Tsig_typext (sub.type_extension sub te)
     | Tsig_exception ext ->
-        Tsig_exception (sub.extension_constructor sub ext)
+        Tsig_exception (sub.type_exception sub ext)
     | Tsig_module x ->
         Tsig_module (sub.module_declaration sub x)
+    | Tsig_modsubst x ->
+        Tsig_modsubst (sub.module_substitution sub x)
     | Tsig_recmodule list ->
         Tsig_recmodule (List.map (sub.module_declaration sub) list)
     | Tsig_modtype x ->
@@ -387,7 +419,7 @@ let signature_item sub x =
     | Tsig_class_type list ->
         Tsig_class_type
           (List.map (sub.class_type_declaration sub) list)
-    | Tsig_open _
+    | Tsig_open od -> Tsig_open (sub.open_description sub od)
     | Tsig_attribute _ as d -> d
   in
   {x with sig_desc; sig_env}
@@ -425,12 +457,19 @@ let with_constraint sub = function
   | Twith_module _
   | Twith_modsubst _ as d -> d
 
+let open_description sub od =
+  {od with open_env = sub.env sub od.open_env}
+
+let open_declaration sub od =
+  {od with open_expr = sub.module_expr sub od.open_expr;
+           open_env = sub.env sub od.open_env}
+
 let module_coercion sub = function
   | Tcoerce_none -> Tcoerce_none
   | Tcoerce_functor (c1,c2) ->
       Tcoerce_functor (sub.module_coercion sub c1, sub.module_coercion sub c2)
-  | Tcoerce_alias (p, c1) ->
-      Tcoerce_alias (p, sub.module_coercion sub c1)
+  | Tcoerce_alias (env, p, c1) ->
+      Tcoerce_alias (sub.env sub env, p, sub.module_coercion sub c1)
   | Tcoerce_structure (l1, l2) ->
       let l1' = List.map (fun (i,c) -> i, sub.module_coercion sub c) l1 in
       let l2' =
@@ -500,7 +539,7 @@ let class_expr sub x =
         Tcl_fun (
           label,
           sub.pat sub pat,
-          List.map (tuple3 id id (sub.expr sub)) priv,
+          List.map (tuple2 id (sub.expr sub)) priv,
           sub.class_expr sub cl,
           partial
         )
@@ -516,11 +555,13 @@ let class_expr sub x =
         Tcl_let (
           rec_flag,
           value_bindings,
-          List.map (tuple3 id id (sub.expr sub)) ivars,
+          List.map (tuple2 id (sub.expr sub)) ivars,
           sub.class_expr sub cl
         )
     | Tcl_ident (path, lid, tyl) ->
         Tcl_ident (path, lid, List.map (sub.typ sub) tyl)
+    | Tcl_open (od, e) ->
+        Tcl_open (sub.open_description sub od, sub.class_expr sub e)
   in
   {x with cl_desc; cl_env}
 
@@ -541,6 +582,8 @@ let class_type sub x =
            sub.typ sub ct,
            sub.class_type sub cl
           )
+    | Tcty_open (od, e) ->
+        Tcty_open (sub.open_description sub od, sub.class_type sub e)
   in
   {x with cltyp_desc; cltyp_env}
 
@@ -576,10 +619,7 @@ let typ sub x =
     | Ttyp_constr (path, lid, list) ->
         Ttyp_constr (path, lid, List.map (sub.typ sub) list)
     | Ttyp_object (list, closed) ->
-        Ttyp_object (
-          List.map (tuple3 id id (sub.typ sub)) list,
-          closed
-        )
+        Ttyp_object ((List.map (sub.object_field sub) list), closed)
     | Ttyp_class (path, lid, list) ->
         Ttyp_class
           (path,
@@ -602,10 +642,21 @@ let class_structure sub x =
   let cstr_fields = List.map (sub.class_field sub) x.cstr_fields in
   {x with cstr_self; cstr_fields}
 
-let row_field sub = function
-  | Ttag (label, attrs, b, list) ->
-      Ttag (label, attrs, b, List.map (sub.typ sub) list)
-  | Tinherit ct -> Tinherit (sub.typ sub ct)
+let row_field sub x =
+  let rf_desc = match x.rf_desc with
+    | Ttag (label, b, list) ->
+        Ttag (label, b, List.map (sub.typ sub) list)
+    | Tinherit ct -> Tinherit (sub.typ sub ct)
+  in
+  { x with rf_desc; }
+
+let object_field sub x =
+  let of_desc = match x.of_desc with
+    | OTtag (label, ct) ->
+        OTtag (label, (sub.typ sub ct))
+    | OTinherit ct -> OTinherit (sub.typ sub ct)
+  in
+  { x with of_desc; }
 
 let class_field_kind sub = function
   | Tcfk_virtual ct -> Tcfk_virtual (sub.typ sub ct)
@@ -653,6 +704,7 @@ let env _sub x = x
 
 let default =
   {
+    binding_op;
     case;
     cases;
     class_declaration;
@@ -670,12 +722,16 @@ let default =
     module_binding;
     module_coercion;
     module_declaration;
+    module_substitution;
     module_expr;
     module_type;
     module_type_declaration;
     package_type;
     pat;
     row_field;
+    object_field;
+    open_declaration;
+    open_description;
     signature;
     signature_item;
     structure;
@@ -684,6 +740,7 @@ let default =
     type_declaration;
     type_declarations;
     type_extension;
+    type_exception;
     type_kind;
     value_binding;
     value_bindings;
