@@ -38,7 +38,7 @@ char *file_prefix = 0;
 char *myname = "yacc";
 char temp_form[] = "yacc.XXXXXXX";
 
-#ifdef NO_UNIX
+#ifdef _WIN32
 char dirsep = '\\';
 #else
 char dirsep = '/';
@@ -56,11 +56,10 @@ char *defines_file_name;
 char *input_file_name = "";
 char *output_file_name;
 char *text_file_name;
-char *union_file_name;
 char *verbose_file_name;
 
 #ifdef HAS_MKSTEMP
-int action_fd = -1, entry_fd = -1, text_fd = -1, union_fd = -1;
+int action_fd = -1, entry_fd = -1, text_fd = -1;
 #endif
 
 FILE *action_file;      /*  a temp file, used to save actions associated    */
@@ -72,9 +71,6 @@ FILE *input_file;       /*  the input file                                  */
 FILE *output_file;      /*  y.tab.c                                         */
 FILE *text_file;        /*  a temp file, used to save text until all        */
                         /*  symbols have been defined                       */
-FILE *union_file;       /*  a temp file, used to save the union             */
-                        /*  definition until all symbol have been           */
-                        /*  defined                                         */
 FILE *verbose_file;     /*  y.output                                        */
 FILE *interface_file;
 
@@ -104,9 +100,6 @@ char *nullable;
 #if !defined(HAS_MKSTEMP)
 extern char *mktemp(char *);
 #endif
-#ifndef NO_UNIX
-extern char *getenv(const char *);
-#endif
 
 
 void done(int k)
@@ -118,13 +111,10 @@ void done(int k)
        unlink(entry_file_name);
     if (text_fd != -1)
        unlink(text_file_name);
-    if (union_fd != -1)
-       unlink(union_file_name);
 #else
     if (action_file) { fclose(action_file); unlink(action_file_name); }
     if (entry_file) { fclose(entry_file); unlink(entry_file_name); }
     if (text_file) { fclose(text_file); unlink(text_file_name); }
-    if (union_file) { fclose(union_file); unlink(union_file_name); }
 #endif
     if (output_file && k > 0) {
       fclose(output_file); unlink(output_file_name);
@@ -284,7 +274,7 @@ void create_file_names(void)
     int i, len;
     char *tmpdir;
 
-#ifdef NO_UNIX
+#ifdef _WIN32
     tmpdir = getenv("TEMP");
     if (tmpdir == 0) tmpdir = ".";
 #else
@@ -302,32 +292,26 @@ void create_file_names(void)
     if (entry_file_name == 0) no_space();
     text_file_name = MALLOC(i);
     if (text_file_name == 0) no_space();
-    union_file_name = MALLOC(i);
-    if (union_file_name == 0) no_space();
 
     strcpy(action_file_name, tmpdir);
     strcpy(entry_file_name, tmpdir);
     strcpy(text_file_name, tmpdir);
-    strcpy(union_file_name, tmpdir);
 
     if (len && tmpdir[len - 1] != dirsep)
     {
         action_file_name[len] = dirsep;
         entry_file_name[len] = dirsep;
         text_file_name[len] = dirsep;
-        union_file_name[len] = dirsep;
         ++len;
     }
 
     strcpy(action_file_name + len, temp_form);
     strcpy(entry_file_name + len, temp_form);
     strcpy(text_file_name + len, temp_form);
-    strcpy(union_file_name + len, temp_form);
 
     action_file_name[len + 5] = 'a';
     entry_file_name[len + 5] = 'e';
     text_file_name[len + 5] = 't';
-    union_file_name[len + 5] = 'u';
 
 #ifdef HAS_MKSTEMP
     action_fd = mkstemp(action_file_name);
@@ -339,14 +323,10 @@ void create_file_names(void)
     text_fd = mkstemp(text_file_name);
     if (text_fd == -1)
         open_error(text_file_name);
-    union_fd = mkstemp(union_file_name);
-    if (union_fd == -1)
-        open_error(union_file_name);
 #else
     mktemp(action_file_name);
     mktemp(entry_file_name);
     mktemp(text_file_name);
-    mktemp(union_file_name);
 #endif
 
     len = strlen(file_prefix);
@@ -424,13 +404,6 @@ void open_files(void)
         defines_file = fopen(defines_file_name, "w");
         if (defines_file == 0)
             open_error(defines_file_name);
-#ifdef HAS_MKSTEMP
-        union_file = fdopen(union_fd, "w");
-#else
-        union_file = fopen(union_file_name, "w");
-#endif
-        if (union_file ==  0)
-            open_error(union_file_name);
     }
 
     output_file = fopen(output_file_name, "w");
